# -*- coding: utf-8 -*-
'''
    :codeauthor: Megan Wilhite<mwilhite@saltstack.com>
'''

# Import Python libs
from __future__ import absolute_import

# Import Salt Libs
import salt.modules.mac_service as mac_service

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import skipIf, TestCase
from tests.support.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)


@skipIf(NO_MOCK, NO_MOCK_REASON)
class MacServiceTestCase(TestCase, LoaderModuleMockMixin):
    '''
    TestCase for salt.modules.mac_service module
    '''
    def setup_loader_modules(self):
        return {mac_service: {}}

    def test_service_disabled_when_enabled(self):
        '''
        test service.disabled when service is enabled
        '''
        srv_name = 'com.apple.atrun'
        cmd = 'disabled services = {\n\t"com.saltstack.salt.minion" => false\n\t"com.apple.atrun" => false\n{'

        with patch.object(mac_service, 'launchctl', MagicMock(return_value=cmd)):
            self.assertFalse(mac_service.disabled(srv_name))

    def test_service_disabled_when_disabled(self):
        '''
        test service.disabled when service is disabled
        '''
        srv_name = 'com.apple.atrun'
        cmd = 'disabled services = {\n\t"com.saltstack.salt.minion" => false\n\t"com.apple.atrun" => true\n{'

        with patch.object(mac_service, 'launchctl', MagicMock(return_value=cmd)):
            self.assertTrue(mac_service.disabled(srv_name))

    def test_service_disabled_srvname_wrong(self):
        '''
        test service.disabled when service is just slightly wrong
        '''
        srv_names = ['com.apple.atru', 'com', 'apple']
        cmd = 'disabled services = {\n\t"com.saltstack.salt.minion" => false\n\t"com.apple.atrun" => true\n}'
        for name in srv_names:
            with patch.object(mac_service, 'launchctl', MagicMock(return_value=cmd)):
                self.assertFalse(mac_service.disabled(name))

    def test_service_disabled_status_upper_case(self):
        '''
        test service.disabled when disabled status is uppercase
        '''
        srv_name = 'com.apple.atrun'
        cmd = 'disabled services = {\n\t"com.saltstack.salt.minion" => false\n\t"com.apple.atrun" => True\n{'

        with patch.object(mac_service, 'launchctl', MagicMock(return_value=cmd)):
            self.assertTrue(mac_service.disabled(srv_name))



# -*- coding: utf-8 -*-
'''
    :codeauthor: Mike Place (mp@saltstack.com)


    tests.unit.returners.smtp_return_test
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'''

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import NO_MOCK, NO_MOCK_REASON, MagicMock, patch

# Import salt libs
import salt.returners.smtp_return as smtp
from salt.utils.jinja import SaltCacheLoader

try:
    import gnupg  # pylint: disable=unused-import
    HAS_GNUPG = True
except ImportError:
    HAS_GNUPG = False


@skipIf(NO_MOCK, NO_MOCK_REASON)
class SMTPReturnerTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test SMTP returner
    '''
    def setup_loader_modules(self):
        return {
            smtp: {
                '__opts__': {
                    'extension_modules': '',
                    'optimization_order': [0, 1, 2],
                    'renderer': 'jinja|yaml',
                    'renderer_blacklist': [],
                    'renderer_whitelist': [],
                    'file_roots': {},
                    'pillar_roots': {},
                    'cachedir': '/',
                    'master_uri': 'tcp://127.0.0.1:4505',
                    'pki_dir': '/',
                    'keysize': 2048,
                    'id': 'test',
                    '__role': 'minion',
                }
            }
        }

    def _test_returner(self, mocked_smtplib):  # pylint: disable=unused-argument
        '''
        Test to see if the SMTP returner sends a message
        '''
        ret = {'id': '12345',
               'fun': 'mytest.func',
               'fun_args': 'myfunc args',
               'jid': '54321',
               'return': 'The room is on fire as shes fixing her hair'}
        options = {'username': '',
                   'tls': '',
                   'from': '',
                   'fields': 'id,fun,fun_args,jid,return',
                   'to': '',
                   'host': '',
                   'renderer': 'jinja|yaml',
                   'template': '',
                   'password': '',
                   'gpgowner': '',
                   'subject': ''}

        with patch('salt.returners.smtp_return._get_options', MagicMock(return_value=options)), \
             patch.object(SaltCacheLoader, 'file_client', MagicMock()):
            smtp.returner(ret)
            self.assertTrue(mocked_smtplib.return_value.sendmail.called)

    if HAS_GNUPG:
        def test_returner(self):
            with patch('salt.returners.smtp_return.gnupg'), \
                    patch('salt.returners.smtp_return.smtplib.SMTP') as mocked_smtplib:
                self._test_returner(mocked_smtplib)

    else:
        def test_returner(self):
            with patch('salt.returners.smtp_return.smtplib.SMTP') as mocked_smtplib:
                self._test_returner(mocked_smtplib)



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import salt module
import salt.modules.mac_pkgutil as mac_pkgutil

# Import Salt Testing libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import NO_MOCK, NO_MOCK_REASON, patch


@skipIf(NO_MOCK, NO_MOCK_REASON)
class MacPkgutilTestCase(TestCase, LoaderModuleMockMixin):

    def setup_loader_modules(self):
        return {mac_pkgutil: {}}

    def test_install(self):
        # Given
        source = '/foo/bar/fubar.pkg'
        package_id = 'com.foo.fubar.pkg'

        # When
        with patch('salt.modules.mac_pkgutil.is_installed',
                   return_value=False):
            with patch('salt.modules.mac_pkgutil._install_from_path',
                       return_value=True) as _install_from_path:
                mac_pkgutil.install(source, package_id)

        # Then
        _install_from_path.assert_called_with(source)

    def test_install_already_there(self):
        # Given
        source = '/foo/bar/fubar.pkg'
        package_id = 'com.foo.fubar.pkg'

        # When
        with patch('salt.modules.mac_pkgutil.is_installed',
                   return_value=True):
            with patch('salt.modules.mac_pkgutil._install_from_path',
                       return_value=True) as _install_from_path:
                mac_pkgutil.install(source, package_id)

        # Then
        self.assertEqual(_install_from_path.called, 0)



# -*- coding: utf-8 -*-
'''
    :codeauthor: Bo Maryniuk <bo@suse.de>
'''

# Import Python Libs
from __future__ import absolute_import

# Import Salt Testing Libs
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

import os
import imp
import sys
from zypp_plugin import BogusIO

if sys.version_info >= (3,):
    BUILTINS_OPEN = 'builtins.open'
else:
    BUILTINS_OPEN = '__builtin__.open'

zyppnotify = imp.load_source('zyppnotify', os.path.sep.join(os.path.dirname(__file__).split(
    os.path.sep)[:-2] + ['scripts', 'suse', 'zypper', 'plugins', 'commit', 'zyppnotify']))


@skipIf(NO_MOCK, NO_MOCK_REASON)
class ZyppPluginsTestCase(TestCase):
    '''
    Test shipped libzypp plugins.
    '''
    def test_drift_detector(self):
        '''
        Test drift detector for a correct cookie file.
        Returns:

        '''
        drift = zyppnotify.DriftDetector()
        drift._get_mtime = MagicMock(return_value=123)
        drift._get_checksum = MagicMock(return_value='deadbeef')
        bogus_io = BogusIO()
        with patch(BUILTINS_OPEN, bogus_io):
            drift.PLUGINEND(None, None)
        self.assertEqual(str(bogus_io), 'deadbeef 123\n')
        self.assertEqual(bogus_io.mode, 'w')
        self.assertEqual(bogus_io.path, '/var/cache/salt/minion/rpmdb.cookie')



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals
from functools import wraps
import os
import io
import stat

# Import Salt libs
import salt.config
import salt.daemons.masterapi as masterapi
import salt.utils.platform

# Import Salt Testing Libs
from tests.support.paths import TMP_CONF_DIR
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    patch,
    MagicMock,
    NO_MOCK,
    NO_MOCK_REASON
)


def gen_permissions(owner='', group='', others=''):
    '''
    Helper method to generate file permission bits
    Usage: gen_permissions('rw', 'r', 'r')
    '''
    ret = 0
    for c in owner:
        ret |= getattr(stat, 'S_I{}USR'.format(c.upper()), 0)
    for c in group:
        ret |= getattr(stat, 'S_I{}GRP'.format(c.upper()), 0)
    for c in others:
        ret |= getattr(stat, 'S_I{}OTH'.format(c.upper()), 0)
    return ret


def patch_check_permissions(uid=1, groups=None, is_windows=False, permissive_pki=False):
    if not groups:
        groups = [uid]

    def decorator(func):
        @wraps(func)
        def wrapper(self):
            self.auto_key.opts['permissive_pki_access'] = permissive_pki
            if salt.utils.platform.is_windows():
                with patch('salt.utils.platform.is_windows', MagicMock(return_value=True)):
                    func(self)
            else:
                with patch('os.stat', self.os_stat_mock), \
                     patch('os.getuid', MagicMock(return_value=uid)), \
                     patch('salt.utils.user.get_gid_list', MagicMock(return_value=groups)), \
                     patch('salt.utils.platform.is_windows', MagicMock(return_value=is_windows)):
                    func(self)
        return wrapper
    return decorator


@skipIf(NO_MOCK, NO_MOCK_REASON)
class AutoKeyTest(TestCase):
    '''
    Tests for the salt.daemons.masterapi.AutoKey class
    '''

    def setUp(self):
        opts = salt.config.master_config(None)
        opts['user'] = 'test_user'
        self.auto_key = masterapi.AutoKey(opts)
        self.stats = {}

    def os_stat_mock(self, filename):
        fmode = MagicMock()
        fstats = self.stats.get(filename, {})
        fmode.st_mode = fstats.get('mode', 0)
        fmode.st_gid = fstats.get('gid', 0)
        return fmode

    @patch_check_permissions(uid=0, is_windows=True)
    def test_check_permissions_windows(self):
        '''
        Assert that all files are accepted on windows
        '''
        self.stats['testfile'] = {'mode': gen_permissions('rwx', 'rwx', 'rwx'), 'gid': 2}
        self.assertTrue(self.auto_key.check_permissions('testfile'))

    @patch_check_permissions(permissive_pki=True)
    def test_check_permissions_others_can_write(self):
        '''
        Assert that no file is accepted, when others can write to it
        '''
        self.stats['testfile'] = {'mode': gen_permissions('', '', 'w'), 'gid': 1}
        if salt.utils.platform.is_windows():
            self.assertTrue(self.auto_key.check_permissions('testfile'))
        else:
            self.assertFalse(self.auto_key.check_permissions('testfile'))

    @patch_check_permissions()
    def test_check_permissions_group_can_write_not_permissive(self):
        '''
        Assert that a file is accepted, when group can write to it and perkissive_pki_access=False
        '''
        self.stats['testfile'] = {'mode': gen_permissions('w', 'w', ''), 'gid': 1}
        if salt.utils.platform.is_windows():
            self.assertTrue(self.auto_key.check_permissions('testfile'))
        else:
            self.assertFalse(self.auto_key.check_permissions('testfile'))

    @patch_check_permissions(permissive_pki=True)
    def test_check_permissions_group_can_write_permissive(self):
        '''
        Assert that a file is accepted, when group can write to it and perkissive_pki_access=True
        '''
        self.stats['testfile'] = {'mode': gen_permissions('w', 'w', ''), 'gid': 1}
        self.assertTrue(self.auto_key.check_permissions('testfile'))

    @patch_check_permissions(uid=0, permissive_pki=True)
    def test_check_permissions_group_can_write_permissive_root_in_group(self):
        '''
        Assert that a file is accepted, when group can write to it, perkissive_pki_access=False,
        salt is root and in the file owning group
        '''
        self.stats['testfile'] = {'mode': gen_permissions('w', 'w', ''), 'gid': 0}
        self.assertTrue(self.auto_key.check_permissions('testfile'))

    @patch_check_permissions(uid=0, permissive_pki=True)
    def test_check_permissions_group_can_write_permissive_root_not_in_group(self):
        '''
        Assert that no file is accepted, when group can write to it, perkissive_pki_access=False,
        salt is root and **not** in the file owning group
        '''
        self.stats['testfile'] = {'mode': gen_permissions('w', 'w', ''), 'gid': 1}
        if salt.utils.platform.is_windows():
            self.assertTrue(self.auto_key.check_permissions('testfile'))
        else:
            self.assertFalse(self.auto_key.check_permissions('testfile'))

    @patch_check_permissions()
    def test_check_permissions_only_owner_can_write(self):
        '''
        Assert that a file is accepted, when only the owner can write to it
        '''
        self.stats['testfile'] = {'mode': gen_permissions('w', '', ''), 'gid': 1}
        self.assertTrue(self.auto_key.check_permissions('testfile'))

    @patch_check_permissions(uid=0)
    def test_check_permissions_only_owner_can_write_root(self):
        '''
        Assert that a file is accepted, when only the owner can write to it and salt is root
        '''
        self.stats['testfile'] = {'mode': gen_permissions('w', '', ''), 'gid': 0}
        self.assertTrue(self.auto_key.check_permissions('testfile'))

    def _test_check_autosign_grains(self,
                                    test_func,
                                    file_content='test_value',
                                    file_name='test_grain',
                                    autosign_grains_dir='test_dir',
                                    permissions_ret=True):
        '''
        Helper function for testing autosign_grains().

        Patches ``os.walk`` to return only ``file_name`` and ``salt.utils.files.fopen`` to open a
        mock file with ``file_content`` as content. Optionally sets ``opts`` values.
        Then executes test_func. The ``os.walk`` and ``salt.utils.files.fopen`` mock objects
        are passed to the function as arguments.
        '''
        if autosign_grains_dir:
            self.auto_key.opts['autosign_grains_dir'] = autosign_grains_dir
        mock_file = io.StringIO(file_content)
        mock_dirs = [(None, None, [file_name])]

        with patch('os.walk', MagicMock(return_value=mock_dirs)) as mock_walk, \
             patch('salt.utils.files.fopen', MagicMock(return_value=mock_file)) as mock_open, \
             patch('salt.daemons.masterapi.AutoKey.check_permissions',
                MagicMock(return_value=permissions_ret)) as mock_permissions:
            test_func(mock_walk, mock_open, mock_permissions)

    def test_check_autosign_grains_no_grains(self):
        '''
        Asserts that autosigning from grains fails when no grain values are passed.
        '''
        def test_func(mock_walk, mock_open, mock_permissions):
            self.assertFalse(self.auto_key.check_autosign_grains(None))
            self.assertEqual(mock_walk.call_count, 0)
            self.assertEqual(mock_open.call_count, 0)
            self.assertEqual(mock_permissions.call_count, 0)

            self.assertFalse(self.auto_key.check_autosign_grains({}))
            self.assertEqual(mock_walk.call_count, 0)
            self.assertEqual(mock_open.call_count, 0)
            self.assertEqual(mock_permissions.call_count, 0)

        self._test_check_autosign_grains(test_func)

    def test_check_autosign_grains_no_autosign_grains_dir(self):
        '''
        Asserts that autosigning from grains fails when the \'autosign_grains_dir\' config option
        is undefined.
        '''
        def test_func(mock_walk, mock_open, mock_permissions):
            self.assertFalse(self.auto_key.check_autosign_grains({'test_grain': 'test_value'}))
            self.assertEqual(mock_walk.call_count, 0)
            self.assertEqual(mock_open.call_count, 0)
            self.assertEqual(mock_permissions.call_count, 0)

        self._test_check_autosign_grains(test_func, autosign_grains_dir=None)

    def test_check_autosign_grains_accept(self):
        '''
        Asserts that autosigning from grains passes when a matching grain value is in an
        autosign_grain file.
        '''
        def test_func(*args):
            self.assertTrue(self.auto_key.check_autosign_grains({'test_grain': 'test_value'}))

        file_content = '#test_ignore\ntest_value'
        self._test_check_autosign_grains(test_func, file_content=file_content)

    def test_check_autosign_grains_accept_not(self):
        '''
        Asserts that autosigning from grains fails when the grain value is not in the
        autosign_grain files.
        '''
        def test_func(*args):
            self.assertFalse(self.auto_key.check_autosign_grains({'test_grain': 'test_invalid'}))

        file_content = '#test_invalid\ntest_value'
        self._test_check_autosign_grains(test_func, file_content=file_content)

    def test_check_autosign_grains_invalid_file_permissions(self):
        '''
        Asserts that autosigning from grains fails when the grain file has the wrong permissions.
        '''
        def test_func(*args):
            self.assertFalse(self.auto_key.check_autosign_grains({'test_grain': 'test_value'}))

        file_content = '#test_ignore\ntest_value'
        self._test_check_autosign_grains(test_func, file_content=file_content, permissions_ret=False)


@skipIf(NO_MOCK, NO_MOCK_REASON)
class LocalFuncsTestCase(TestCase):
    '''
    TestCase for salt.daemons.masterapi.LocalFuncs class
    '''

    def setUp(self):
        opts = salt.config.master_config(None)
        self.local_funcs = masterapi.LocalFuncs(opts, 'test-key')

    # runner tests

    def test_runner_token_not_authenticated(self):
        '''
        Asserts that a TokenAuthenticationError is returned when the token can't authenticate.
        '''
        mock_ret = {'error': {'name': 'TokenAuthenticationError',
                              'message': 'Authentication failure of type "token" occurred.'}}
        ret = self.local_funcs.runner({'token': 'asdfasdfasdfasdf'})
        self.assertDictEqual(mock_ret, ret)

    def test_runner_token_authorization_error(self):
        '''
        Asserts that a TokenAuthenticationError is returned when the token authenticates, but is
        not authorized.
        '''
        token = 'asdfasdfasdfasdf'
        load = {'token': token, 'fun': 'test.arg', 'kwarg': {}}
        mock_token = {'token': token, 'eauth': 'foo', 'name': 'test'}
        mock_ret = {'error': {'name': 'TokenAuthenticationError',
                              'message': 'Authentication failure of type "token" occurred '
                                         'for user test.'}}

        with patch('salt.auth.LoadAuth.authenticate_token', MagicMock(return_value=mock_token)), \
             patch('salt.auth.LoadAuth.get_auth_list', MagicMock(return_value=[])):
            ret = self.local_funcs.runner(load)

        self.assertDictEqual(mock_ret, ret)

    def test_runner_token_salt_invocation_error(self):
        '''
        Asserts that a SaltInvocationError is returned when the token authenticates, but the
        command is malformed.
        '''
        token = 'asdfasdfasdfasdf'
        load = {'token': token, 'fun': 'badtestarg', 'kwarg': {}}
        mock_token = {'token': token, 'eauth': 'foo', 'name': 'test'}
        mock_ret = {'error': {'name': 'SaltInvocationError',
                              'message': 'A command invocation error occurred: Check syntax.'}}

        with patch('salt.auth.LoadAuth.authenticate_token', MagicMock(return_value=mock_token)), \
             patch('salt.auth.LoadAuth.get_auth_list', MagicMock(return_value=['testing'])):
            ret = self.local_funcs.runner(load)

        self.assertDictEqual(mock_ret, ret)

    def test_runner_eauth_not_authenticated(self):
        '''
        Asserts that an EauthAuthenticationError is returned when the user can't authenticate.
        '''
        mock_ret = {'error': {'name': 'EauthAuthenticationError',
                              'message': 'Authentication failure of type "eauth" occurred for '
                                         'user UNKNOWN.'}}
        ret = self.local_funcs.runner({'eauth': 'foo'})
        self.assertDictEqual(mock_ret, ret)

    def test_runner_eauth_authorization_error(self):
        '''
        Asserts that an EauthAuthenticationError is returned when the user authenticates, but is
        not authorized.
        '''
        load = {'eauth': 'foo', 'username': 'test', 'fun': 'test.arg', 'kwarg': {}}
        mock_ret = {'error': {'name': 'EauthAuthenticationError',
                              'message': 'Authentication failure of type "eauth" occurred for '
                                         'user test.'}}
        with patch('salt.auth.LoadAuth.authenticate_eauth', MagicMock(return_value=True)), \
             patch('salt.auth.LoadAuth.get_auth_list', MagicMock(return_value=[])):
            ret = self.local_funcs.runner(load)

        self.assertDictEqual(mock_ret, ret)

    def test_runner_eauth_salt_invocation_error(self):
        '''
        Asserts that an EauthAuthenticationError is returned when the user authenticates, but the
        command is malformed.
        '''
        load = {'eauth': 'foo', 'username': 'test', 'fun': 'bad.test.arg.func', 'kwarg': {}}
        mock_ret = {'error': {'name': 'SaltInvocationError',
                              'message': 'A command invocation error occurred: Check syntax.'}}
        with patch('salt.auth.LoadAuth.authenticate_eauth', MagicMock(return_value=True)), \
             patch('salt.auth.LoadAuth.get_auth_list', MagicMock(return_value=['testing'])):
            ret = self.local_funcs.runner(load)

        self.assertDictEqual(mock_ret, ret)

    # wheel tests

    def test_wheel_token_not_authenticated(self):
        '''
        Asserts that a TokenAuthenticationError is returned when the token can't authenticate.
        '''
        mock_ret = {'error': {'name': 'TokenAuthenticationError',
                              'message': 'Authentication failure of type "token" occurred.'}}
        ret = self.local_funcs.wheel({'token': 'asdfasdfasdfasdf'})
        self.assertDictEqual(mock_ret, ret)

    def test_wheel_token_authorization_error(self):
        '''
        Asserts that a TokenAuthenticationError is returned when the token authenticates, but is
        not authorized.
        '''
        token = 'asdfasdfasdfasdf'
        load = {'token': token, 'fun': 'test.arg', 'kwarg': {}}
        mock_token = {'token': token, 'eauth': 'foo', 'name': 'test'}
        mock_ret = {'error': {'name': 'TokenAuthenticationError',
                              'message': 'Authentication failure of type "token" occurred '
                                         'for user test.'}}

        with patch('salt.auth.LoadAuth.authenticate_token', MagicMock(return_value=mock_token)), \
             patch('salt.auth.LoadAuth.get_auth_list', MagicMock(return_value=[])):
            ret = self.local_funcs.wheel(load)

        self.assertDictEqual(mock_ret, ret)

    def test_wheel_token_salt_invocation_error(self):
        '''
        Asserts that a SaltInvocationError is returned when the token authenticates, but the
        command is malformed.
        '''
        token = 'asdfasdfasdfasdf'
        load = {'token': token, 'fun': 'badtestarg', 'kwarg': {}}
        mock_token = {'token': token, 'eauth': 'foo', 'name': 'test'}
        mock_ret = {'error': {'name': 'SaltInvocationError',
                              'message': 'A command invocation error occurred: Check syntax.'}}

        with patch('salt.auth.LoadAuth.authenticate_token', MagicMock(return_value=mock_token)), \
             patch('salt.auth.LoadAuth.get_auth_list', MagicMock(return_value=['testing'])):
            ret = self.local_funcs.wheel(load)

        self.assertDictEqual(mock_ret, ret)

    def test_wheel_eauth_not_authenticated(self):
        '''
        Asserts that an EauthAuthenticationError is returned when the user can't authenticate.
        '''
        mock_ret = {'error': {'name': 'EauthAuthenticationError',
                              'message': 'Authentication failure of type "eauth" occurred for '
                                         'user UNKNOWN.'}}
        ret = self.local_funcs.wheel({'eauth': 'foo'})
        self.assertDictEqual(mock_ret, ret)

    def test_wheel_eauth_authorization_error(self):
        '''
        Asserts that an EauthAuthenticationError is returned when the user authenticates, but is
        not authorized.
        '''
        load = {'eauth': 'foo', 'username': 'test', 'fun': 'test.arg', 'kwarg': {}}
        mock_ret = {'error': {'name': 'EauthAuthenticationError',
                              'message': 'Authentication failure of type "eauth" occurred for '
                                         'user test.'}}
        with patch('salt.auth.LoadAuth.authenticate_eauth', MagicMock(return_value=True)), \
             patch('salt.auth.LoadAuth.get_auth_list', MagicMock(return_value=[])):
            ret = self.local_funcs.wheel(load)

        self.assertDictEqual(mock_ret, ret)

    def test_wheel_eauth_salt_invocation_error(self):
        '''
        Asserts that an EauthAuthenticationError is returned when the user authenticates, but the
        command is malformed.
        '''
        load = {'eauth': 'foo', 'username': 'test', 'fun': 'bad.test.arg.func', 'kwarg': {}}
        mock_ret = {'error': {'name': 'SaltInvocationError',
                              'message': 'A command invocation error occurred: Check syntax.'}}
        with patch('salt.auth.LoadAuth.authenticate_eauth', MagicMock(return_value=True)), \
             patch('salt.auth.LoadAuth.get_auth_list', MagicMock(return_value=['testing'])):
            ret = self.local_funcs.wheel(load)

        self.assertDictEqual(mock_ret, ret)

    def test_wheel_user_not_authenticated(self):
        '''
        Asserts that an UserAuthenticationError is returned when the user can't authenticate.
        '''
        mock_ret = {'error': {'name': 'UserAuthenticationError',
                              'message': 'Authentication failure of type "user" occurred for '
                                         'user UNKNOWN.'}}
        ret = self.local_funcs.wheel({})
        self.assertDictEqual(mock_ret, ret)

    # publish tests

    def test_publish_user_is_blacklisted(self):
        '''
        Asserts that an AuthorizationError is returned when the user has been blacklisted.
        '''
        mock_ret = {'error': {'name': 'AuthorizationError',
                              'message': 'Authorization error occurred.'}}
        with patch('salt.acl.PublisherACL.user_is_blacklisted', MagicMock(return_value=True)):
            self.assertEqual(mock_ret, self.local_funcs.publish({'user': 'foo', 'fun': 'test.arg'}))

    def test_publish_cmd_blacklisted(self):
        '''
        Asserts that an AuthorizationError is returned when the command has been blacklisted.
        '''
        mock_ret = {'error': {'name': 'AuthorizationError',
                              'message': 'Authorization error occurred.'}}
        with patch('salt.acl.PublisherACL.user_is_blacklisted', MagicMock(return_value=False)), \
                patch('salt.acl.PublisherACL.cmd_is_blacklisted', MagicMock(return_value=True)):
            self.assertEqual(mock_ret, self.local_funcs.publish({'user': 'foo', 'fun': 'test.arg'}))

    def test_publish_token_not_authenticated(self):
        '''
        Asserts that an AuthenticationError is returned when the token can't authenticate.
        '''
        load = {'user': 'foo', 'fun': 'test.arg', 'tgt': 'test_minion',
                'kwargs': {'token': 'asdfasdfasdfasdf'}}
        mock_ret = {'error': {'name': 'AuthenticationError',
                              'message': 'Authentication error occurred.'}}
        with patch('salt.acl.PublisherACL.user_is_blacklisted', MagicMock(return_value=False)), \
                patch('salt.acl.PublisherACL.cmd_is_blacklisted', MagicMock(return_value=False)):
            self.assertEqual(mock_ret, self.local_funcs.publish(load))

    def test_publish_token_authorization_error(self):
        '''
        Asserts that an AuthorizationError is returned when the token authenticates, but is not
        authorized.
        '''
        token = 'asdfasdfasdfasdf'
        load = {'user': 'foo', 'fun': 'test.arg', 'tgt': 'test_minion',
                'arg': 'bar', 'kwargs': {'token': token}}
        mock_token = {'token': token, 'eauth': 'foo', 'name': 'test'}
        mock_ret = {'error': {'name': 'AuthorizationError',
                              'message': 'Authorization error occurred.'}}

        with patch('salt.acl.PublisherACL.user_is_blacklisted', MagicMock(return_value=False)), \
                patch('salt.acl.PublisherACL.cmd_is_blacklisted', MagicMock(return_value=False)), \
                patch('salt.auth.LoadAuth.authenticate_token', MagicMock(return_value=mock_token)), \
                patch('salt.auth.LoadAuth.get_auth_list', MagicMock(return_value=[])):
            self.assertEqual(mock_ret, self.local_funcs.publish(load))

    def test_publish_eauth_not_authenticated(self):
        '''
        Asserts that an AuthenticationError is returned when the user can't authenticate.
        '''
        load = {'user': 'test', 'fun': 'test.arg', 'tgt': 'test_minion',
                'kwargs': {'eauth': 'foo'}}
        mock_ret = {'error': {'name': 'AuthenticationError',
                              'message': 'Authentication error occurred.'}}
        with patch('salt.acl.PublisherACL.user_is_blacklisted', MagicMock(return_value=False)), \
                patch('salt.acl.PublisherACL.cmd_is_blacklisted', MagicMock(return_value=False)):
            self.assertEqual(mock_ret, self.local_funcs.publish(load))

    def test_publish_eauth_authorization_error(self):
        '''
        Asserts that an AuthorizationError is returned when the user authenticates, but is not
        authorized.
        '''
        load = {'user': 'test', 'fun': 'test.arg', 'tgt': 'test_minion',
                'kwargs': {'eauth': 'foo'}, 'arg': 'bar'}
        mock_ret = {'error': {'name': 'AuthorizationError',
                              'message': 'Authorization error occurred.'}}
        with patch('salt.acl.PublisherACL.user_is_blacklisted', MagicMock(return_value=False)), \
                patch('salt.acl.PublisherACL.cmd_is_blacklisted', MagicMock(return_value=False)), \
                patch('salt.auth.LoadAuth.authenticate_eauth', MagicMock(return_value=True)), \
                patch('salt.auth.LoadAuth.get_auth_list', MagicMock(return_value=[])):
            self.assertEqual(mock_ret, self.local_funcs.publish(load))

    def test_publish_user_not_authenticated(self):
        '''
        Asserts that an AuthenticationError is returned when the user can't authenticate.
        '''
        load = {'user': 'test', 'fun': 'test.arg', 'tgt': 'test_minion'}
        mock_ret = {'error': {'name': 'AuthenticationError',
                              'message': 'Authentication error occurred.'}}
        with patch('salt.acl.PublisherACL.user_is_blacklisted', MagicMock(return_value=False)), \
                patch('salt.acl.PublisherACL.cmd_is_blacklisted', MagicMock(return_value=False)):
            self.assertEqual(mock_ret, self.local_funcs.publish(load))

    def test_publish_user_authenticated_missing_auth_list(self):
        '''
        Asserts that an AuthenticationError is returned when the user has an effective user id and is
        authenticated, but the auth_list is empty.
        '''
        load = {'user': 'test', 'fun': 'test.arg', 'tgt': 'test_minion',
                'kwargs': {'user': 'test'}, 'arg': 'foo'}
        mock_ret = {'error': {'name': 'AuthenticationError',
                              'message': 'Authentication error occurred.'}}
        with patch('salt.acl.PublisherACL.user_is_blacklisted', MagicMock(return_value=False)), \
                patch('salt.acl.PublisherACL.cmd_is_blacklisted', MagicMock(return_value=False)), \
                patch('salt.auth.LoadAuth.authenticate_key', MagicMock(return_value='fake-user-key')), \
                patch('salt.utils.master.get_values_of_matching_keys', MagicMock(return_value=[])):
            self.assertEqual(mock_ret, self.local_funcs.publish(load))

    def test_publish_user_authorization_error(self):
        '''
        Asserts that an AuthorizationError is returned when the user authenticates, but is not
        authorized.
        '''
        load = {'user': 'test', 'fun': 'test.arg', 'tgt': 'test_minion',
                'kwargs': {'user': 'test'}, 'arg': 'foo'}
        mock_ret = {'error': {'name': 'AuthorizationError',
                              'message': 'Authorization error occurred.'}}
        with patch('salt.acl.PublisherACL.user_is_blacklisted', MagicMock(return_value=False)), \
                patch('salt.acl.PublisherACL.cmd_is_blacklisted', MagicMock(return_value=False)), \
                patch('salt.auth.LoadAuth.authenticate_key', MagicMock(return_value='fake-user-key')), \
                patch('salt.utils.master.get_values_of_matching_keys', MagicMock(return_value=['test'])), \
                patch('salt.utils.minions.CkMinions.auth_check', MagicMock(return_value=False)):
            self.assertEqual(mock_ret, self.local_funcs.publish(load))


class FakeCache(object):

    def __init__(self):
        self.data = {}

    def store(self, bank, key, value):
        self.data[bank, key] = value

    def fetch(self, bank, key):
        return self.data[bank, key]


class RemoteFuncsTestCase(TestCase):
    '''
    TestCase for salt.daemons.masterapi.RemoteFuncs class
    '''

    def setUp(self):
        opts = salt.config.master_config(os.path.join(TMP_CONF_DIR, 'master'))
        self.funcs = masterapi.RemoteFuncs(opts)
        self.funcs.cache = FakeCache()

    def test_mine_get(self, tgt_type_key='tgt_type'):
        '''
        Asserts that ``mine_get`` gives the expected results.

        Actually this only tests that:

        - the correct check minions method is called
        - the correct cache key is subsequently used
        '''
        self.funcs.cache.store('minions/webserver', 'mine',
                               dict(ip_addr='2001:db8::1:3'))
        with patch('salt.utils.minions.CkMinions._check_compound_minions',
                   MagicMock(return_value=(dict(
                       minions=['webserver'],
                       missing=[])))):
            ret = self.funcs._mine_get(
                {
                    'id': 'requester_minion',
                    'tgt': 'G@roles:web',
                    'fun': 'ip_addr',
                    tgt_type_key: 'compound',
                }
            )
        self.assertDictEqual(ret, dict(webserver='2001:db8::1:3'))

    def test_mine_get_pre_nitrogen_compat(self):
        '''
        Asserts that pre-Nitrogen API key ``expr_form`` is still accepted.

        This is what minions before Nitrogen would issue.
        '''
        self.test_mine_get(tgt_type_key='expr_form')



# -*- coding: utf-8 -*-
'''
    :codeauthor: Megan Wilhite <mwilhite@saltstack.com>
'''

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals
import os

# Import Salt Libs
from salt.config import cloud_providers_config, cloud_config
from salt.ext import six

# Import Salt Testing LIbs
from tests.support.case import ShellCase
from tests.support.paths import FILES
from tests.support.helpers import expensiveTest, generate_random_name

# Create the cloud instance name to be used throughout the tests
INSTANCE_NAME = generate_random_name('CLOUD-TEST-')
PROVIDER_NAME = 'vmware'
TIMEOUT = 500


class VMWareTest(ShellCase):
    '''
    Integration tests for the vmware cloud provider in Salt-Cloud
    '''

    @expensiveTest
    def setUp(self):
        '''
        Sets up the test requirements
        '''

        # check if appropriate cloud provider and profile files are present
        profile_str = 'vmware-config'
        providers = self.run_cloud('--list-providers')

        if profile_str + ':' not in providers:
            self.skipTest(
                'Configuration file for {0} was not found. Check {0}.conf files '
                'in tests/integration/files/conf/cloud.*.d/ to run these tests.'
                .format(PROVIDER_NAME)
            )

        # check if user, password, url and provider are present
        config = cloud_providers_config(
            os.path.join(
                FILES,
                'conf',
                'cloud.providers.d',
                PROVIDER_NAME + '.conf'
            )
        )

        user = config[profile_str][PROVIDER_NAME]['user']
        password = config[profile_str][PROVIDER_NAME]['password']
        url = config[profile_str][PROVIDER_NAME]['url']

        conf_items = [user, password, url]
        missing_conf_item = []

        for item in conf_items:
            if item == '':
                missing_conf_item.append(item)

        if missing_conf_item:
            self.skipTest(
                'A user, password, and url must be provided to run these tests.'
                'One or more of these elements is missing. Check'
                'tests/integration/files/conf/cloud.providers.d/{0}.conf'
                .format(PROVIDER_NAME)
            )

    def test_instance(self):
        '''
        Tests creating and deleting an instance on vmware and installing salt
        '''
        # create the instance
        profile = os.path.join(
                FILES,
                'conf',
                'cloud.profiles.d',
                PROVIDER_NAME + '.conf'
            )

        profile_config = cloud_config(profile)
        disk_datastore = profile_config['vmware-test']['devices']['disk']['Hard disk 2']['datastore']

        instance = self.run_cloud('-p vmware-test {0}'.format(INSTANCE_NAME), timeout=TIMEOUT)
        ret_str = '{0}:'.format(INSTANCE_NAME)
        disk_datastore_str = '                [{0}] {1}/Hard disk 2-flat.vmdk'.format(disk_datastore, INSTANCE_NAME)

        # check if instance returned with salt installed
        try:
            self.assertIn(ret_str, instance)
            self.assertIn(disk_datastore_str, instance,
                          msg='Hard Disk 2 did not use the Datastore {0} '.format(disk_datastore))
        except AssertionError:
            self.run_cloud('-d {0} --assume-yes'.format(INSTANCE_NAME), timeout=TIMEOUT)
            raise

        # delete the instance
        delete = self.run_cloud('-d {0} --assume-yes'.format(INSTANCE_NAME), timeout=TIMEOUT)
        ret_str = '{0}:\', \'            True'.format(INSTANCE_NAME)

        # check if deletion was performed appropriately
        self.assertIn(ret_str, six.text_type(delete))

    def test_snapshot(self):
        '''
        Tests creating snapshot and creating vm with --no-deploy
        '''
        # create the instance
        instance = self.run_cloud('-p vmware-test {0} --no-deploy'.format(INSTANCE_NAME),
                                  timeout=TIMEOUT)
        ret_str = '{0}:'.format(INSTANCE_NAME)

        # check if instance returned with salt installed
        try:
            self.assertIn(ret_str, instance)
        except AssertionError:
            self.run_cloud('-d {0} --assume-yes'.format(INSTANCE_NAME), timeout=TIMEOUT)
            raise

        create_snapshot = self.run_cloud('-a create_snapshot {0} \
                                         snapshot_name=\'Test Cloud\' \
                                         memdump=True -y'.format(INSTANCE_NAME),
                                         timeout=TIMEOUT)
        s_ret_str = 'Snapshot created successfully'

        self.assertIn(s_ret_str, six.text_type(create_snapshot))

        # delete the instance
        delete = self.run_cloud('-d {0} --assume-yes'.format(INSTANCE_NAME), timeout=TIMEOUT)
        ret_str = '{0}:\', \'            True'.format(INSTANCE_NAME)

        self.assertIn(ret_str, six.text_type(delete))

    def tearDown(self):
        '''
        Clean up after tests
        '''
        query = self.run_cloud('--query')
        ret_str = '        {0}:'.format(INSTANCE_NAME)

        # if test instance is still present, delete it
        if ret_str in query:
            self.run_cloud('-d {0} --assume-yes'.format(INSTANCE_NAME), timeout=TIMEOUT)



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals
import re

# Import Salt Testing libs
from tests.support.case import ModuleCase
from tests.support.unit import skipIf

# Import Salt libs
import salt.utils.platform


@skipIf(not salt.utils.platform.is_windows(), 'windows test only')
class WinIPTest(ModuleCase):
    '''
    Tests for salt.modules.win_ip
    '''
    def test_get_default_gateway(self):
        '''
        Test getting default gateway
        '''
        ip = re.compile(r'^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$')
        ret = self.run_function('ip.get_default_gateway')
        assert ip.match(ret)

    def test_ip_is_enabled(self):
        '''
        Test ip.is_enabled
        '''
        assert self.run_function('ip.is_enabled', ['Ethernet'])
        assert 'not found' in self.run_function('ip.is_enabled', ['doesnotexist'])



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing libs
from tests.support.case import ModuleCase
from tests.support.unit import skipIf
from tests.support.helpers import (
    requires_salt_modules,
    destructiveTest,
)

# Import Salt libs
import salt.utils.platform


def _find_new_locale(current_locale):
    for locale in ['en_US.UTF-8', 'de_DE.UTF-8', 'fr_FR.UTF-8']:
        if locale != current_locale:
            return locale


@skipIf(salt.utils.platform.is_windows(), 'minion is windows')
@skipIf(salt.utils.platform.is_darwin(), 'locale method is not supported on mac')
@requires_salt_modules('locale')
class LocaleModuleTest(ModuleCase):
    def test_get_locale(self):
        locale = self.run_function('locale.get_locale')
        self.assertNotIn('Unsupported platform!', locale)

    @destructiveTest
    def test_gen_locale(self):
        # Make sure charmaps are available on test system before attempting
        # call gen_locale. We log this error to the user in the function, but
        # we don't want to fail this test if this is missing on the test system.
        char_maps = self.run_function('cmd.run_all', ['locale -m'])
        if char_maps['stdout'] == '':
            self.skipTest('locale charmaps not available. Skipping test.')

        if char_maps['retcode'] and char_maps['stderr']:
            self.skipTest('{0}. Cannot generate locale. Skipping test.'.format(
                char_maps['stderr'])
            )

        locale = self.run_function('locale.get_locale')
        new_locale = _find_new_locale(locale)
        ret = self.run_function('locale.gen_locale', [new_locale])
        self.assertTrue(ret)

    @destructiveTest
    def test_set_locale(self):
        original_locale = self.run_function('locale.get_locale')
        locale_to_set = _find_new_locale(original_locale)
        self.run_function('locale.gen_locale', [locale_to_set])
        ret = self.run_function('locale.set_locale', [locale_to_set])
        new_locale = self.run_function('locale.get_locale')
        self.assertTrue(ret)
        self.assertEqual(locale_to_set, new_locale)
        self.run_function('locale.set_locale', [original_locale])



# -*- coding: utf-8 -*-
'''
Tests for the salt-run command
'''
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing libs
from tests.support.case import ShellCase

import logging
log = logging.getLogger(__name__)


class ManageTest(ShellCase):
    '''
    Test the manage runner
    '''
    def test_cache(self):
        '''
        Store, list, fetch, then flush data
        '''
        # Store the data
        ret = self.run_run_plus(
            'cache.store',
            bank='cachetest/runner',
            key='test_cache',
            data='The time has come the walrus said',
        )
        # Make sure we can see the new key
        ret = self.run_run_plus('cache.list', bank='cachetest/runner')
        self.assertIn('test_cache', ret['return'])
        # Make sure we can see the new data
        ret = self.run_run_plus('cache.fetch', bank='cachetest/runner', key='test_cache')
        self.assertIn('The time has come the walrus said', ret['return'])
        # Make sure we can delete the data
        ret = self.run_run_plus('cache.flush', bank='cachetest/runner', key='test_cache')
        ret = self.run_run_plus('cache.list', bank='cachetest/runner')
        self.assertNotIn('test_cache', ret['return'])

    def test_grains(self):
        '''
        Test cache.grains
        '''
        # Store the data
        ret = self.run_run_plus(
            'cache.grains',
            tgt='minion'
        )

        self.assertIn('minion', ret['return'])

    def test_pillar(self):
        '''
        Test cache.pillar
        '''
        # Store the data
        ret = self.run_run_plus(
            'cache.pillar',
            tgt='minion'
        )

        self.assertIn('minion', ret['return'])

    def test_mine(self):
        '''
        Test cache.mine
        '''
        # Store the data
        ret = self.run_run_plus(
            'cache.mine',
            tgt='minion'
        )

        self.assertIn('minion', ret['return'])



# -*- coding: utf-8 -*-
'''
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
'''
# Import Python libs
from __future__ import absolute_import, unicode_literals, print_function
import sys

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import skipIf, TestCase
from tests.support.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    MagicMock,
    patch)


# Import Salt Libs
import salt.states.linux_acl as linux_acl
from salt.exceptions import CommandExecutionError


@skipIf(NO_MOCK, NO_MOCK_REASON)
@skipIf(not sys.platform.startswith('linux'), 'Test for Linux only')
class LinuxAclTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.states.linux_acl
    '''
    def setup_loader_modules(self):
        return {linux_acl: {}}

    # 'present' function tests: 1

    def test_present(self):
        '''
        Test to ensure a Linux ACL is present
        '''
        self.maxDiff = None
        name = '/root'
        acl_type = 'users'
        acl_name = 'damian'
        perms = 'rwx'

        mock = MagicMock(side_effect=[{name: {acl_type: [{acl_name:
                                                         {'octal': 'A'}}]}},
                                      {name: {acl_type: [{acl_name:
                                                         {'octal': 'A'}}]}},
                                      {name: {acl_type: [{acl_name:
                                                         {'octal': 'A'}}]}},
                                      {name: {acl_type: [{}]}},
                                      {name: {acl_type: [{}]}},
                                      {name: {acl_type: [{}]}},
                                      {
                                          name: {acl_type: [{acl_name: {'octal': 7}}]},
                                          name+"/foo": {acl_type: [{acl_name: {'octal': 'A'}}]}
                                      },
                                      {
                                          name: {acl_type: [{acl_name: {'octal': 7}}]},
                                          name+"/foo": {acl_type: [{acl_name: {'octal': 7}}]}
                                      },
                                      {name: {acl_type: ''}}])
        mock_modfacl = MagicMock(return_value=True)

        with patch.dict(linux_acl.__salt__, {'acl.getfacl': mock}):
            # Update - test=True
            with patch.dict(linux_acl.__opts__, {'test': True}):
                comt = ('Updated permissions will be applied for {0}: A -> {1}'
                        ''.format(acl_name, perms))
                ret = {'name': name,
                       'comment': comt,
                       'changes': {},
                       'pchanges': {'new': {'acl_name': acl_name,
                                            'acl_type': acl_type,
                                            'perms': perms},
                                    'old': {'acl_name': acl_name,
                                            'acl_type': acl_type,
                                            'perms': 'A'}},
                       'result': None}

                self.assertDictEqual(linux_acl.present(name, acl_type, acl_name,
                                                       perms), ret)
            # Update - test=False
            with patch.dict(linux_acl.__salt__, {'acl.modfacl': mock_modfacl}):
                with patch.dict(linux_acl.__opts__, {'test': False}):
                    comt = ('Updated permissions for {0}'.format(acl_name))
                    ret = {'name': name,
                           'comment': comt,
                           'changes': {'new': {'acl_name': acl_name,
                                               'acl_type': acl_type,
                                               'perms': perms},
                                       'old': {'acl_name': acl_name,
                                               'acl_type': acl_type,
                                               'perms': 'A'}},
                           'pchanges': {},
                           'result': True}
                    self.assertDictEqual(linux_acl.present(name, acl_type,
                                                           acl_name, perms),
                                         ret)
            # Update - modfacl error
            with patch.dict(linux_acl.__salt__, {'acl.modfacl': MagicMock(
                    side_effect=CommandExecutionError('Custom err'))}):
                with patch.dict(linux_acl.__opts__, {'test': False}):
                    comt = ('Error updating permissions for {0}: Custom err'
                            ''.format(acl_name))
                    ret = {'name': name,
                           'comment': comt,
                           'changes': {},
                           'pchanges': {},
                           'result': False}
                    self.assertDictEqual(linux_acl.present(name, acl_type,
                                                           acl_name, perms),
                                         ret)
            # New - test=True
            with patch.dict(linux_acl.__salt__, {'acl.modfacl': mock_modfacl}):
                with patch.dict(linux_acl.__opts__, {'test': True}):
                    comt = ('New permissions will be applied '
                            'for {0}: {1}'.format(acl_name, perms))
                    ret = {'name': name,
                           'comment': comt,
                           'changes': {},
                           'pchanges': {'new': {'acl_name': acl_name,
                                                'acl_type': acl_type,
                                                'perms': perms}},
                           'result': None}
                    self.assertDictEqual(linux_acl.present(name, acl_type,
                                                           acl_name, perms),
                                         ret)
            # New - test=False
            with patch.dict(linux_acl.__salt__, {'acl.modfacl': mock_modfacl}):
                with patch.dict(linux_acl.__opts__, {'test': False}):
                    comt = ('Applied new permissions for {0}'.format(acl_name))
                    ret = {'name': name,
                           'comment': comt,
                           'changes': {'new': {'acl_name': acl_name,
                                               'acl_type': acl_type,
                                               'perms': perms}},
                           'pchanges': {},
                           'result': True}
                    self.assertDictEqual(linux_acl.present(name, acl_type,
                                                           acl_name, perms),
                                         ret)
            # New - modfacl error
            with patch.dict(linux_acl.__salt__, {'acl.modfacl': MagicMock(
                    side_effect=CommandExecutionError('Custom err'))}):
                with patch.dict(linux_acl.__opts__, {'test': False}):
                    comt = ('Error updating permissions for {0}: Custom err'
                            ''.format(acl_name))
                    ret = {'name': name,
                           'comment': comt,
                           'changes': {},
                           'pchanges': {},
                           'result': False}
                    self.assertDictEqual(linux_acl.present(name, acl_type,
                                                           acl_name, perms),
                                         ret)

            # New - recurse true
            with patch.dict(linux_acl.__salt__, {'acl.getfacl': mock}):
                # Update - test=True
                with patch.dict(linux_acl.__opts__, {'test': True}):
                    comt = ('Updated permissions will be applied for {0}: 7 -> {1}'
                            ''.format(acl_name, perms))
                    ret = {'name': name,
                           'comment': comt,
                           'changes': {},
                           'pchanges': {'new': {'acl_name': acl_name,
                                                'acl_type': acl_type,
                                                'perms': perms},
                                        'old': {'acl_name': acl_name,
                                                'acl_type': acl_type,
                                                'perms': '7'}},
                           'result': None}

                    self.assertDictEqual(linux_acl.present(name, acl_type, acl_name,
                                                           perms, recurse=False), ret)

            # New - recurse true - nothing to do
            with patch.dict(linux_acl.__salt__, {'acl.getfacl': mock}):
                # Update - test=True
                with patch.dict(linux_acl.__opts__, {'test': True}):
                    comt = ('Permissions are in the desired state')
                    ret = {'name': name,
                           'comment': comt,
                           'changes': {},
                           'pchanges': {},
                           'result': True}

                    self.assertDictEqual(linux_acl.present(name, acl_type, acl_name,
                                                           perms, recurse=True), ret)

            # No acl type
            comt = ('ACL Type does not exist')
            ret = {'name': name, 'comment': comt, 'result': False,
                   'changes': {}, 'pchanges': {}}
            self.assertDictEqual(linux_acl.present(name, acl_type, acl_name,
                                                   perms), ret)

    # 'absent' function tests: 2

    def test_absent(self):
        '''
        Test to ensure a Linux ACL does not exist
        '''
        name = '/root'
        acl_type = 'users'
        acl_name = 'damian'
        perms = 'rwx'

        ret = {'name': name,
               'result': None,
               'comment': '',
               'changes': {}}

        mock = MagicMock(side_effect=[{name: {acl_type: [{acl_name: {'octal': 'A'}}]}},
                                      {name: {acl_type: ''}}])
        with patch.dict(linux_acl.__salt__, {'acl.getfacl': mock}):
            with patch.dict(linux_acl.__opts__, {'test': True}):
                comt = ('Removing permissions')
                ret.update({'comment': comt})
                self.assertDictEqual(linux_acl.absent(name, acl_type, acl_name, perms), ret)

            comt = ('ACL Type does not exist')
            ret.update({'comment': comt, 'result': False})
            self.assertDictEqual(linux_acl.absent(name, acl_type, acl_name, perms), ret)

    def test_absent_recursive(self):
        '''
        Test to ensure a Linux ACL does not exist
        '''
        name = '/root'
        acl_type = 'users'
        acl_name = 'damian'
        perms = 'rwx'

        ret = {'name': name,
               'result': None,
               'comment': '',
               'changes': {}}

        mock = MagicMock(side_effect=[
            {name: {acl_type: [{acl_name: {'octal': 7}}]}, name+"/foo": {acl_type: [{acl_name: {'octal': 'A'}}]}}
        ])
        with patch.dict(linux_acl.__salt__, {'acl.getfacl': mock}):
            with patch.dict(linux_acl.__opts__, {'test': True}):
                comt = ('Removing permissions')
                ret.update({'comment': comt})
                self.assertDictEqual(linux_acl.absent(name, acl_type, acl_name, perms, recurse=True), ret)



# -*- coding: utf-8 -*-

# Import pytohn libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt testing libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import skipIf, TestCase
from tests.support.mock import NO_MOCK, NO_MOCK_REASON, MagicMock, patch

# Import Salt libs
import salt.modules.artifactory as artifactory


@skipIf(NO_MOCK, NO_MOCK_REASON)
class ArtifactoryTestCase(TestCase, LoaderModuleMockMixin):

    def setup_loader_modules(self):
        return {artifactory: {}}

    def test_artifact_get_metadata(self):
        with patch('salt.modules.artifactory._get_artifact_metadata_xml', MagicMock(return_value='''<?xml version="1.0" encoding="UTF-8"?>
                <metadata>
                  <groupId>com.company.sampleapp.web-module</groupId>
                  <artifactId>web</artifactId>
                  <versioning>
                    <latest>1.1_RC11</latest>
                    <release>1.0.1</release>
                    <versions>
                      <version>1.0_RC20</version>
                      <version>1.0_RC22</version>
                    </versions>
                    <lastUpdated>20140623120632</lastUpdated>
                  </versioning>
                </metadata>
            ''')):
            metadata = artifactory._get_artifact_metadata(artifactory_url='http://artifactory.example.com/artifactory',
                                                repository='libs-releases',
                                                group_id='com.company.sampleapp.web-module',
                                                artifact_id='web',
                                                headers={})
            self.assertEqual(metadata['latest_version'], '1.1_RC11')

    def test_snapshot_version_get_metadata(self):
        with patch('salt.modules.artifactory._get_snapshot_version_metadata_xml', MagicMock(return_value='''<?xml version="1.0" encoding="UTF-8"?>
                    <metadata>
                      <groupId>com.company.sampleapp.web-module</groupId>
                      <artifactId>web</artifactId>
                      <version>1.1_RC8-SNAPSHOT</version>
                      <versioning>
                        <snapshot>
                          <timestamp>20140418.150212</timestamp>
                          <buildNumber>1</buildNumber>
                        </snapshot>
                        <lastUpdated>20140623104055</lastUpdated>
                        <snapshotVersions>
                          <snapshotVersion>
                            <extension>pom</extension>
                            <value>1.1_RC8-20140418.150212-1</value>
                            <updated>20140418150212</updated>
                          </snapshotVersion>
                          <snapshotVersion>
                            <extension>war</extension>
                            <value>1.1_RC8-20140418.150212-1</value>
                            <updated>20140418150212</updated>
                          </snapshotVersion>
                        </snapshotVersions>
                      </versioning>
                    </metadata>
                ''')):
            metadata = artifactory._get_snapshot_version_metadata(artifactory_url='http://artifactory.example.com/artifactory',
                                                                 repository='libs-releases',
                                                                 group_id='com.company.sampleapp.web-module',
                                                                 artifact_id='web',
                                                                 version='1.1_RC8-SNAPSHOT',
                                                                 headers={})
            self.assertEqual(metadata['snapshot_versions']['war'], '1.1_RC8-20140418.150212-1')

    def test_artifact_metadata_url(self):
        metadata_url = artifactory._get_artifact_metadata_url(artifactory_url='http://artifactory.example.com/artifactory',
                                                             repository='libs-releases',
                                                             group_id='com.company.sampleapp.web-module',
                                                             artifact_id='web')

        self.assertEqual(metadata_url, "http://artifactory.example.com/artifactory/libs-releases/com/company/sampleapp/web-module/web/maven-metadata.xml")

    def test_snapshot_version_metadata_url(self):
        metadata_url = artifactory._get_snapshot_version_metadata_url(artifactory_url='http://artifactory.example.com/artifactory',
                                                             repository='libs-snapshots',
                                                             group_id='com.company.sampleapp.web-module',
                                                             artifact_id='web',
                                                             version='1.0_RC10-SNAPSHOT')

        self.assertEqual(metadata_url, "http://artifactory.example.com/artifactory/libs-snapshots/com/company/sampleapp/web-module/web/1.0_RC10-SNAPSHOT/maven-metadata.xml")

    def test_construct_url_for_released_version(self):
        artifact_url, file_name = artifactory._get_release_url(repository='libs-releases',
                                      group_id='com.company.sampleapp.web-module',
                                      artifact_id='web',
                                      packaging='war',
                                      artifactory_url='http://artifactory.example.com/artifactory',
                                      version='1.0_RC20')

        self.assertEqual(artifact_url, "http://artifactory.example.com/artifactory/libs-releases/com/company/sampleapp/web-module/web/1.0_RC20/web-1.0_RC20.war")
        self.assertEqual(file_name, "web-1.0_RC20.war")

    def test_construct_url_for_snapshot_version(self):
        with patch('salt.modules.artifactory._get_snapshot_version_metadata', MagicMock(return_value={'snapshot_versions': {'war': '1.0_RC10-20131127.105838-2'}})):

            artifact_url, file_name = artifactory._get_snapshot_url(artifactory_url='http://artifactory.example.com/artifactory',
                                                   repository='libs-snapshots',
                                                   group_id='com.company.sampleapp.web-module',
                                                   artifact_id='web',
                                                   version='1.0_RC10-SNAPSHOT',
                                                   packaging='war',
                                                   headers={})

            self.assertEqual(artifact_url, "http://artifactory.example.com/artifactory/libs-snapshots/com/company/sampleapp/web-module/web/1.0_RC10-SNAPSHOT/web-1.0_RC10-20131127.105838-2.war")
            self.assertEqual(file_name, "web-1.0_RC10-20131127.105838-2.war")



# -*- coding: utf-8 -*-
'''
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
'''
# Import Python libs
from __future__ import absolute_import, unicode_literals, print_function

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import skipIf, TestCase
from tests.support.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    MagicMock,
    patch
)

# Import Salt Libs
import salt.states.rabbitmq_vhost as rabbitmq_vhost


@skipIf(NO_MOCK, NO_MOCK_REASON)
class RabbitmqVhostTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.states.rabbitmq_vhost
    '''
    def setup_loader_modules(self):
        return {rabbitmq_vhost: {}}

    # 'present' function tests: 1

    def test_present(self):
        '''
        Test to ensure the RabbitMQ VHost exists.
        '''
        name = 'virtual_host'

        ret = {'name': name,
               'changes': {'new': 'virtual_host', 'old': ''},
               'result': None,
               'comment': "Virtual Host 'virtual_host' will be created."}

        mock = MagicMock(return_value=False)
        with patch.dict(rabbitmq_vhost.__salt__,
                        {'rabbitmq.vhost_exists': mock}):
            with patch.dict(rabbitmq_vhost.__opts__, {'test': True}):
                self.assertDictEqual(rabbitmq_vhost.present(name), ret)

    # 'absent' function tests: 1

    def test_absent(self):
        '''
        Test to ensure the named user is absent.
        '''
        name = 'myqueue'

        ret = {'name': name,
               'changes': {},
               'result': True,
               'comment': "Virtual Host '{0}' is not present.".format(name)}

        mock = MagicMock(return_value=False)
        with patch.dict(rabbitmq_vhost.__salt__,
                        {'rabbitmq.vhost_exists': mock}):
            self.assertDictEqual(rabbitmq_vhost.absent(name), ret)



# -*- coding: utf-8 -*-
'''
    :codeauthor: Anthony Shaw <anthonyshaw@apache.org>
'''

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    MagicMock,
    NO_MOCK,
    NO_MOCK_REASON
)
import salt.modules.servicenow as servicenow


class MockServiceNowClient(object):
    def __init__(self, instance_name, username, password):
        pass

    def get(self, query):
        return [{'query_size': len(query),
                 'query_value': query}]


@skipIf(NO_MOCK, NO_MOCK_REASON)
class ServiceNowModuleTestCase(TestCase, LoaderModuleMockMixin):
    def setup_loader_modules(self):
        module_globals = {
            'Client': MockServiceNowClient,
            '__salt__': {
                'config.option': MagicMock(return_value={
                    'instance_name': 'test',
                    'username': 'mr_test',
                    'password': 'test123'
                })
            }
        }
        if servicenow.HAS_LIBS is False:
            module_globals['sys.modules'] = {'servicenow_rest': MagicMock()}
            module_globals['sys.modules']['servicenow_rest'].api.Client = MockServiceNowClient
        return {servicenow: module_globals}

    def test_module_creation(self):
        client = servicenow._get_client()
        self.assertFalse(client is None)

    def test_non_structured_query(self):
        result = servicenow.non_structured_query('tests', 'role=web')
        self.assertFalse(result is None)
        self.assertEqual(result[0]['query_size'], 8)
        self.assertEqual(result[0]['query_value'], 'role=web')

    def test_non_structured_query_kwarg(self):
        result = servicenow.non_structured_query('tests', role='web')
        self.assertFalse(result is None)
        self.assertEqual(result[0]['query_size'], 8)
        self.assertEqual(result[0]['query_value'], 'role=web')

    def test_non_structured_query_kwarg_multi(self):
        result = servicenow.non_structured_query('tests', role='web',
                                                 type='computer')
        self.assertFalse(result is None)
        self.assertEqual(result[0]['query_size'], 22)



# -*- coding: utf-8 -*-
'''
Unit tests for the Snapper module

:codeauthor:    Duncan Mac-Vicar P. <dmacvicar@suse.de>
:codeauthor:    Pablo Suárez Hernández <psuarezhernandez@suse.de>
'''

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals
import sys

# Import Salt Testing libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    MagicMock,
    patch,
    mock_open,
)

# Import Salt libs
from salt.ext import six
from salt.exceptions import CommandExecutionError
import salt.modules.snapper as snapper

DBUS_RET = {
    'ListSnapshots': [
        [42, 1, 0, 1457006571,
         0, 'Some description', '',
         {'userdata1': 'userval1', 'salt_jid': '20160607130930720112'}],
        [43, 2, 42, 1457006572,
         0, 'Blah Blah', '',
         {'userdata2': 'userval2', 'salt_jid': '20160607130930720112'}]
    ],
    'ListConfigs': [
        ['root', '/', {
            'SUBVOLUME': '/', 'NUMBER_MIN_AGE': '1800',
            'TIMELINE_LIMIT_YEARLY': '4-10', 'NUMBER_LIMIT_IMPORTANT': '10',
            'FSTYPE': 'btrfs', 'TIMELINE_LIMIT_MONTHLY': '4-10',
            'ALLOW_GROUPS': '', 'EMPTY_PRE_POST_MIN_AGE': '1800',
            'EMPTY_PRE_POST_CLEANUP': 'yes', 'BACKGROUND_COMPARISON': 'yes',
            'TIMELINE_LIMIT_HOURLY': '4-10', 'ALLOW_USERS': '',
            'TIMELINE_LIMIT_WEEKLY': '0', 'TIMELINE_CREATE': 'no',
            'NUMBER_CLEANUP': 'yes', 'TIMELINE_CLEANUP': 'yes',
            'SPACE_LIMIT': '0.5', 'NUMBER_LIMIT': '10',
            'TIMELINE_MIN_AGE': '1800', 'TIMELINE_LIMIT_DAILY': '4-10',
            'SYNC_ACL': 'no', 'QGROUP': '1/0'}
        ]
    ],
    'GetFiles': [
        ['/root/.viminfo', 8],
        ['/tmp/foo', 52],
        ['/tmp/foo2', 1],
        ['/tmp/foo3', 2],
        ['/var/log/snapper.log', 8],
        ['/var/cache/salt/minion/extmods/modules/snapper.py', 8],
        ['/var/cache/salt/minion/extmods/modules/snapper.pyc', 8],
    ],
}

FILE_CONTENT = {
    '/tmp/foo': {
        "pre": "dummy text",
        "post": "another foobar"
    },
    '/tmp/foo2': {
        "post": "another foobar"
    }
}

MODULE_RET = {
    'SNAPSHOTS': [
        {
            'userdata': {'userdata1': 'userval1', 'salt_jid': '20160607130930720112'},
            'description': 'Some description', 'timestamp': 1457006571,
            'cleanup': '', 'user': 'root', 'type': 'pre', 'id': 42
        },
        {
            'pre': 42,
            'userdata': {'userdata2': 'userval2', 'salt_jid': '20160607130930720112'},
            'description': 'Blah Blah', 'timestamp': 1457006572,
            'cleanup': '', 'user': 'root', 'type': 'post', 'id': 43
        }
    ],
    'LISTCONFIGS': {
        'root': {
            'SUBVOLUME': '/', 'NUMBER_MIN_AGE': '1800',
            'TIMELINE_LIMIT_YEARLY': '4-10', 'NUMBER_LIMIT_IMPORTANT': '10',
            'FSTYPE': 'btrfs', 'TIMELINE_LIMIT_MONTHLY': '4-10',
            'ALLOW_GROUPS': '', 'EMPTY_PRE_POST_MIN_AGE': '1800',
            'EMPTY_PRE_POST_CLEANUP': 'yes', 'BACKGROUND_COMPARISON': 'yes',
            'TIMELINE_LIMIT_HOURLY': '4-10', 'ALLOW_USERS': '',
            'TIMELINE_LIMIT_WEEKLY': '0', 'TIMELINE_CREATE': 'no',
            'NUMBER_CLEANUP': 'yes', 'TIMELINE_CLEANUP': 'yes',
            'SPACE_LIMIT': '0.5', 'NUMBER_LIMIT': '10',
            'TIMELINE_MIN_AGE': '1800', 'TIMELINE_LIMIT_DAILY': '4-10',
            'SYNC_ACL': 'no', 'QGROUP': '1/0'
        }
    },
    'GETFILES': {
        '/root/.viminfo': {'status': ['modified']},
        '/tmp/foo': {'status': ['type changed', 'permission changed', 'owner changed']},
        '/tmp/foo2': {'status': ['created']},
        '/tmp/foo3': {'status': ['deleted']},
        '/var/log/snapper.log': {'status': ['modified']},
        '/var/cache/salt/minion/extmods/modules/snapper.py': {'status': ['modified']},
        '/var/cache/salt/minion/extmods/modules/snapper.pyc': {'status': ['modified']},
    },
    'DIFF': {
        '/tmp/foo': {
            'comment': 'text file changed',
            'diff': "--- /.snapshots/55/snapshot/tmp/foo\n"
                    "+++ /tmp/foo\n"
                    "@@ -1 +1 @@\n"
                    "-dummy text"
                    "+another foobar"
        },
        '/tmp/foo2': {
            'comment': 'text file created',
            'diff': "--- /.snapshots/55/snapshot/tmp/foo2\n"
                    "+++ /tmp/foo2\n"
                    "@@ -0,0 +1 @@\n"
                    "+another foobar",
        },
        '/tmp/foo26': {
            'comment': 'text file created',
            'diff': "--- /.snapshots/55/snapshot/tmp/foo2 \n"
                    "+++ /tmp/foo2 \n"
                    "@@ -1,0 +1,1 @@\n"
                    "+another foobar",
        },
        '/tmp/foo3': {
            'comment': 'binary file changed',
            'old_sha256_digest': 'e61f8b762d83f3b4aeb3689564b0ffbe54fa731a69a1e208dc9440ce0f69d19b',
            'new_sha256_digest': 'f18f971f1517449208a66589085ddd3723f7f6cefb56c141e3d97ae49e1d87fa',
        }
    }
}


@skipIf(sys.platform.startswith('win'), 'Snapper not available on Windows')
@skipIf(NO_MOCK, NO_MOCK_REASON)
class SnapperTestCase(TestCase, LoaderModuleMockMixin):

    def setup_loader_modules(self):

        class DBusException(BaseException):
            get_dbus_name = 'foo'

        dbus_mock = MagicMock()
        dbus_mock.configure_mock(DBusException=DBusException)
        return {snapper: {'dbus': dbus_mock, 'snapper': MagicMock()}}

    def test__snapshot_to_data(self):
        data = snapper._snapshot_to_data(DBUS_RET['ListSnapshots'][0])  # pylint: disable=protected-access
        self.assertEqual(data['id'], 42)
        self.assertNotIn('pre', data)
        self.assertEqual(data['type'], 'pre')
        self.assertEqual(data['user'], 'root')
        self.assertEqual(data['timestamp'], 1457006571)
        self.assertEqual(data['description'], 'Some description')
        self.assertEqual(data['cleanup'], '')
        self.assertEqual(data['userdata']['userdata1'], 'userval1')

    def test_list_snapshots(self):
        with patch('salt.modules.snapper.snapper.ListSnapshots', MagicMock(return_value=DBUS_RET['ListSnapshots'])):
            self.assertEqual(snapper.list_snapshots(), MODULE_RET["SNAPSHOTS"])

    def test_get_snapshot(self):
        with patch('salt.modules.snapper.snapper.GetSnapshot', MagicMock(return_value=DBUS_RET['ListSnapshots'][0])):
            self.assertEqual(snapper.get_snapshot(), MODULE_RET["SNAPSHOTS"][0])
            self.assertEqual(snapper.get_snapshot(number=42), MODULE_RET["SNAPSHOTS"][0])
            self.assertNotEqual(snapper.get_snapshot(number=42), MODULE_RET["SNAPSHOTS"][1])

    def test_list_configs(self):
        with patch('salt.modules.snapper.snapper.ListConfigs', MagicMock(return_value=DBUS_RET['ListConfigs'])):
            self.assertEqual(snapper.list_configs(), MODULE_RET["LISTCONFIGS"])

    def test_get_config(self):
        with patch('salt.modules.snapper.snapper.GetConfig', MagicMock(return_value=DBUS_RET['ListConfigs'][0])):
            self.assertEqual(snapper.get_config(), DBUS_RET["ListConfigs"][0])

    def test_set_config(self):
        with patch('salt.modules.snapper.snapper.SetConfig', MagicMock()):
            opts = {'sync_acl': True, 'dummy': False, 'foobar': 1234}
            self.assertEqual(snapper.set_config(opts), True)

    def test_status_to_string(self):
        self.assertEqual(snapper.status_to_string(1), ["created"])
        self.assertEqual(snapper.status_to_string(2), ["deleted"])
        self.assertEqual(snapper.status_to_string(4), ["type changed"])
        self.assertEqual(snapper.status_to_string(8), ["modified"])
        self.assertEqual(snapper.status_to_string(16), ["permission changed"])
        self.assertListEqual(snapper.status_to_string(24), ["modified", "permission changed"])
        self.assertEqual(snapper.status_to_string(32), ["owner changed"])
        self.assertEqual(snapper.status_to_string(64), ["group changed"])
        self.assertListEqual(snapper.status_to_string(97), ["created", "owner changed", "group changed"])
        self.assertEqual(snapper.status_to_string(128), ["extended attributes changed"])
        self.assertEqual(snapper.status_to_string(256), ["ACL info changed"])

    def test_create_config(self):
        with patch('salt.modules.snapper.snapper.CreateConfig', MagicMock()), \
                patch('salt.modules.snapper.snapper.GetConfig', MagicMock(return_value=DBUS_RET['ListConfigs'][0])):
            opts = {
                'name': 'testconfig',
                'subvolume': '/foo/bar/',
                'fstype': 'btrfs',
                'template': 'mytemplate',
                'extra_opts': {"NUMBER_CLEANUP": False},
            }
            with patch('salt.modules.snapper.set_config', MagicMock()) as set_config_mock:
                self.assertEqual(snapper.create_config(**opts), DBUS_RET['ListConfigs'][0])
                set_config_mock.assert_called_with("testconfig", **opts['extra_opts'])

            with patch('salt.modules.snapper.set_config', MagicMock()) as set_config_mock:
                del opts['extra_opts']
                self.assertEqual(snapper.create_config(**opts), DBUS_RET['ListConfigs'][0])
                assert not set_config_mock.called
                self.assertRaises(CommandExecutionError, snapper.create_config)

    def test_create_snapshot(self):
        with patch('salt.modules.snapper.snapper.CreateSingleSnapshot', MagicMock(return_value=1234)), \
                patch('salt.modules.snapper.snapper.CreatePreSnapshot', MagicMock(return_value=1234)), \
                patch('salt.modules.snapper.snapper.CreatePostSnapshot', MagicMock(return_value=1234)):
            for snapshot_type in ['pre', 'post', 'single']:
                opts = {
                    '__pub_jid': 20160607130930720112,
                    'type': snapshot_type,
                    'description': 'Test description',
                    'cleanup_algorithm': 'number',
                    'pre_number': 23,
                }
                self.assertEqual(snapper.create_snapshot(**opts), 1234)

    def test_delete_snapshot_id_success(self):
        with patch('salt.modules.snapper.snapper.DeleteSnapshots', MagicMock()), \
                patch('salt.modules.snapper.snapper.ListSnapshots', MagicMock(return_value=DBUS_RET['ListSnapshots'])):
            self.assertEqual(snapper.delete_snapshot(snapshots_ids=43), {"root": {"ids": [43], "status": "deleted"}})
            self.assertEqual(snapper.delete_snapshot(snapshots_ids=[42, 43]),
                             {"root": {"ids": [42, 43], "status": "deleted"}})

    def test_delete_snapshot_id_fail(self):
        with patch('salt.modules.snapper.snapper.DeleteSnapshots', MagicMock()), \
                patch('salt.modules.snapper.snapper.ListSnapshots', MagicMock(return_value=DBUS_RET['ListSnapshots'])):
            self.assertRaises(CommandExecutionError, snapper.delete_snapshot)
            self.assertRaises(CommandExecutionError, snapper.delete_snapshot, snapshots_ids=1)
            self.assertRaises(CommandExecutionError, snapper.delete_snapshot, snapshots_ids=[1, 2])

    def test_modify_snapshot(self):
        with patch('salt.modules.snapper.snapper.SetSnapshot', MagicMock()):
            _ret = {
                'userdata': {'userdata2': 'uservalue2'},
                'description': 'UPDATED DESCRIPTION', 'timestamp': 1457006571,
                'cleanup': 'number', 'user': 'root', 'type': 'pre', 'id': 42
            }
            _opts = {
                'config': 'root',
                'snapshot_id': 42,
                'cleanup': 'number',
                'description': 'UPDATED DESCRIPTION',
                'userdata': {'userdata2': 'uservalue2'},
            }
            with patch('salt.modules.snapper.get_snapshot', MagicMock(side_effect=[DBUS_RET['ListSnapshots'][0], _ret])):
                self.assertDictEqual(snapper.modify_snapshot(**_opts), _ret)

    def test__get_num_interval(self):
        with patch('salt.modules.snapper._get_last_snapshot', MagicMock(return_value={'id': 42})):
            self.assertEqual(snapper._get_num_interval(config=None, num_pre=None, num_post=None), (42, 0))  # pylint: disable=protected-access
            self.assertEqual(snapper._get_num_interval(config=None, num_pre=None, num_post=50), (42, 50))  # pylint: disable=protected-access
            self.assertEqual(snapper._get_num_interval(config=None, num_pre=42, num_post=50), (42, 50))  # pylint: disable=protected-access

    def test_run(self):
        patch_dict = {
            'snapper.create_snapshot': MagicMock(return_value=43),
            'test.ping': MagicMock(return_value=True),
        }
        with patch.dict(snapper.__salt__, patch_dict):
            self.assertEqual(snapper.run("test.ping"), True)
            self.assertRaises(CommandExecutionError, snapper.run, "unknown.func")

    def test_status(self):
        with patch('salt.modules.snapper._get_num_interval', MagicMock(return_value=(42, 43))), \
                patch('salt.modules.snapper.snapper.GetComparison', MagicMock()), \
                patch('salt.modules.snapper.snapper.GetFiles', MagicMock(return_value=DBUS_RET['GetFiles'])), \
                patch('salt.modules.snapper.snapper.ListConfigs', MagicMock(return_value=DBUS_RET['ListConfigs'])):
            if six.PY3:
                self.assertCountEqual(snapper.status(), MODULE_RET['GETFILES'])
                self.assertCountEqual(snapper.status(num_pre="42", num_post=43), MODULE_RET['GETFILES'])
                self.assertCountEqual(snapper.status(num_pre=42), MODULE_RET['GETFILES'])
                self.assertCountEqual(snapper.status(num_post=43), MODULE_RET['GETFILES'])
            else:
                self.assertItemsEqual(snapper.status(), MODULE_RET['GETFILES'])
                self.assertItemsEqual(snapper.status(num_pre="42", num_post=43), MODULE_RET['GETFILES'])
                self.assertItemsEqual(snapper.status(num_pre=42), MODULE_RET['GETFILES'])
                self.assertItemsEqual(snapper.status(num_post=43), MODULE_RET['GETFILES'])

    def test_changed_files(self):
        with patch('salt.modules.snapper.status', MagicMock(return_value=MODULE_RET['GETFILES'])):
            self.assertEqual(snapper.changed_files(), MODULE_RET['GETFILES'].keys())

    def test_undo(self):
        with patch('salt.modules.snapper._get_num_interval', MagicMock(return_value=(42, 43))), \
                patch('salt.modules.snapper.status', MagicMock(return_value=MODULE_RET['GETFILES'])):
            cmd_ret = 'create:0 modify:1 delete:0'
            with patch.dict(snapper.__salt__, {'cmd.run': MagicMock(return_value=cmd_ret)}):
                module_ret = {'create': '0', 'delete': '0', 'modify': '1'}
                self.assertEqual(snapper.undo(files=['/tmp/foo']), module_ret)

            cmd_ret = 'create:1 modify:1 delete:0'
            with patch.dict(snapper.__salt__, {'cmd.run': MagicMock(return_value=cmd_ret)}):
                module_ret = {'create': '1', 'delete': '0', 'modify': '1'}
                self.assertEqual(snapper.undo(files=['/tmp/foo', '/tmp/foo2']), module_ret)

            cmd_ret = 'create:1 modify:1 delete:1'
            with patch.dict(snapper.__salt__, {'cmd.run': MagicMock(return_value=cmd_ret)}):
                module_ret = {'create': '1', 'delete': '1', 'modify': '1'}
                self.assertEqual(snapper.undo(files=['/tmp/foo', '/tmp/foo2', '/tmp/foo3']), module_ret)

    def test__get_jid_snapshots(self):
        with patch('salt.modules.snapper.list_snapshots', MagicMock(return_value=MODULE_RET['SNAPSHOTS'])):
            self.assertEqual(
                snapper._get_jid_snapshots("20160607130930720112"),  # pylint: disable=protected-access
                (MODULE_RET['SNAPSHOTS'][0]['id'], MODULE_RET['SNAPSHOTS'][1]['id'])
            )

    def test_undo_jid(self):
        with patch('salt.modules.snapper._get_jid_snapshots', MagicMock(return_value=(42, 43))), \
                patch('salt.modules.snapper.undo', MagicMock(return_value='create:1 modify:1 delete:1')):
            self.assertEqual(snapper.undo_jid(20160607130930720112), 'create:1 modify:1 delete:1')

    def test_diff_text_file(self):
        with patch('salt.modules.snapper._get_num_interval', MagicMock(return_value=(42, 43))), \
                patch('salt.modules.snapper.snapper.MountSnapshot',
                      MagicMock(side_effect=["/.snapshots/55/snapshot", ""])), \
                patch('salt.modules.snapper.snapper.UmountSnapshot', MagicMock(return_value="")), \
                patch('os.path.isdir', MagicMock(return_value=False)), \
                patch('salt.modules.snapper.changed_files', MagicMock(return_value=["/tmp/foo2"])), \
                patch('salt.modules.snapper._is_text_file', MagicMock(return_value=True)), \
                patch('os.path.isfile', MagicMock(side_effect=[False, True])), \
                patch('salt.utils.files.fopen', mock_open(read_data=FILE_CONTENT["/tmp/foo2"]['post'])), \
                patch('salt.modules.snapper.snapper.ListConfigs', MagicMock(return_value=DBUS_RET['ListConfigs'])):
            if sys.version_info < (2, 7):
                self.assertEqual(snapper.diff(), {"/tmp/foo2": MODULE_RET['DIFF']['/tmp/foo26']})
            else:
                self.assertEqual(snapper.diff(), {"/tmp/foo2": MODULE_RET['DIFF']['/tmp/foo2']})

    @skipIf(sys.version_info < (2, 7), 'Python 2.7 required to compare diff properly')
    def test_diff_text_files(self):
        with patch('salt.modules.snapper._get_num_interval', MagicMock(return_value=(55, 0))), \
                patch('salt.modules.snapper.snapper.MountSnapshot',
                      MagicMock(side_effect=["/.snapshots/55/snapshot", "", "/.snapshots/55/snapshot", ""])), \
                patch('salt.modules.snapper.snapper.UmountSnapshot', MagicMock(return_value="")), \
                patch('salt.modules.snapper.changed_files', MagicMock(return_value=["/tmp/foo", "/tmp/foo2"])), \
                patch('salt.modules.snapper._is_text_file', MagicMock(return_value=True)), \
                patch('os.path.isfile', MagicMock(side_effect=[True, True, False, True])), \
                patch('os.path.isdir', MagicMock(return_value=False)), \
                patch('salt.modules.snapper.snapper.ListConfigs', MagicMock(return_value=DBUS_RET['ListConfigs'])):
            contents = {
                '*/tmp/foo': [
                    FILE_CONTENT['/tmp/foo']['pre'],
                    FILE_CONTENT['/tmp/foo']['post'],
                ],
                '*/tmp/foo2': FILE_CONTENT['/tmp/foo2']['post'],
            }
            with patch('salt.utils.files.fopen', mock_open(read_data=contents)):
                module_ret = {
                    "/tmp/foo": MODULE_RET['DIFF']["/tmp/foo"],
                    "/tmp/foo2": MODULE_RET['DIFF']["/tmp/foo2"],
                }
                self.assertEqual(snapper.diff(), module_ret)

    def test_diff_binary_files(self):
        with patch('salt.modules.snapper._get_num_interval', MagicMock(return_value=(55, 0))), \
                patch('salt.modules.snapper.snapper.MountSnapshot',
                      MagicMock(side_effect=["/.snapshots/55/snapshot", "", "/.snapshots/55/snapshot", ""])), \
                patch('salt.modules.snapper.snapper.UmountSnapshot', MagicMock(return_value="")), \
                patch('salt.modules.snapper.changed_files', MagicMock(return_value=["/tmp/foo3"])), \
                patch('salt.modules.snapper._is_text_file', MagicMock(return_value=False)), \
                patch('os.path.isfile', MagicMock(side_effect=[True, True])), \
                patch('os.path.isdir', MagicMock(return_value=False)), \
                patch('salt.modules.snapper.snapper.ListConfigs', MagicMock(return_value=DBUS_RET['ListConfigs'])), \
                patch.dict(snapper.__salt__, {
                        'hashutil.sha256_digest': MagicMock(side_effect=[
                            "e61f8b762d83f3b4aeb3689564b0ffbe54fa731a69a1e208dc9440ce0f69d19b",
                            "f18f971f1517449208a66589085ddd3723f7f6cefb56c141e3d97ae49e1d87fa",
                        ])
                    }):
            with patch('salt.utils.files.fopen', mock_open(read_data='dummy binary')):
                module_ret = {
                    "/tmp/foo3": MODULE_RET['DIFF']["/tmp/foo3"],
                }
                self.assertEqual(snapper.diff(), module_ret)



# -*- coding: utf-8 -*-
'''
unit tests for the slack engine
'''
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import skipIf, TestCase
from tests.support.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    MagicMock,
    patch)

# Import Salt Libs
import salt.engines.slack as slack
import salt.config


@skipIf(slack.HAS_SLACKCLIENT is False, 'The SlackClient is not installed')
@skipIf(NO_MOCK, NO_MOCK_REASON)
class EngineSlackTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.engine.sqs_events
    '''

    def setup_loader_modules(self):
        return {slack: {}}

    def setUp(self):
        mock_opts = salt.config.DEFAULT_MINION_OPTS
        token = 'xoxb-xxxxxxxxxx-xxxxxxxxxxxxxxxxxxxxxxxx'

        with patch.dict(slack.__opts__, mock_opts):
            with patch('slackclient.SlackClient.rtm_connect', MagicMock(return_value=True)):
                self.client = slack.SlackClient(token)

    def test_control_message_target(self):
        '''
        Test slack engine: control_message_target
        '''
        trigger_string = '!'

        loaded_groups = {'default': {'targets': {},
                                     'commands': set(['cmd.run', 'test.ping']),
                                     'default_target': {'tgt_type': 'glob', 'target': '*'},
                                     'users': set(['gareth']),
                                     'aliases': {'whoami': {'cmd': 'cmd.run whoami'},
                                                 'list_pillar': {'cmd': 'pillar.items'}}
                                     }}

        slack_user_name = 'gareth'

        # Check for correct cmdline
        _expected = (True,
                     {'tgt_type': 'glob', 'target': '*'},
                     ['cmd.run', 'whoami'])
        text = '!cmd.run whoami'
        target_commandline = self.client.control_message_target(slack_user_name,
                                                                text,
                                                                loaded_groups,
                                                                trigger_string)

        self.assertEqual(target_commandline, _expected)

        # Check aliases result in correct cmdline
        text = '!whoami'
        target_commandline = self.client.control_message_target(slack_user_name,
                                                                text,
                                                                loaded_groups,
                                                                trigger_string)

        self.assertEqual(target_commandline, _expected)

        # Check pillar is overrided
        _expected = (True,
                     {'tgt_type': 'glob', 'target': '*'},
                     ['pillar.items', 'pillar={"hello": "world"}'])
        text = r"""!list_pillar pillar='{"hello": "world"}'"""
        target_commandline = self.client.control_message_target(slack_user_name,
                                                                text,
                                                                loaded_groups,
                                                                trigger_string)

        self.assertEqual(target_commandline, _expected)

        # Check target is overrided
        _expected = (True,
                     {'tgt_type': 'glob', 'target': 'localhost'},
                     ['cmd.run', 'whoami'])
        text = "!cmd.run whoami target='localhost'"
        target_commandline = self.client.control_message_target(slack_user_name,
                                                                text,
                                                                loaded_groups,
                                                                trigger_string)

        self.assertEqual(target_commandline, _expected)



# -*- coding: utf-8 -*-
'''
Base classes for gitfs/git_pillar integration tests
'''

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals
import copy
import errno
import logging
import os
import psutil
import shutil
import signal
import tempfile
import textwrap
import time

# Import Salt libs
import salt.utils.files
import salt.utils.path
import salt.utils.yaml
from salt.fileserver import gitfs
from salt.pillar import git_pillar
from salt.ext.six.moves import range  # pylint: disable=redefined-builtin

# Import Salt Testing libs
from tests.support.case import ModuleCase
from tests.support.mixins import LoaderModuleMockMixin, SaltReturnAssertsMixin
from tests.support.paths import TMP
from tests.support.helpers import (
    get_unused_localhost_port,
    requires_system_grains,
)
from tests.support.mock import patch

log = logging.getLogger(__name__)

USERNAME = 'gitpillaruser'
PASSWORD = 'saltrules'

_OPTS = {
    '__role': 'minion',
    'environment': None,
    'pillarenv': None,
    'hash_type': 'sha256',
    'file_roots': {},
    'state_top': 'top.sls',
    'state_top_saltenv': None,
    'renderer': 'yaml_jinja',
    'renderer_whitelist': [],
    'renderer_blacklist': [],
    'pillar_merge_lists': False,
    'git_pillar_base': 'master',
    'git_pillar_branch': 'master',
    'git_pillar_env': '',
    'git_pillar_root': '',
    'git_pillar_ssl_verify': True,
    'git_pillar_global_lock': True,
    'git_pillar_user': '',
    'git_pillar_password': '',
    'git_pillar_insecure_auth': False,
    'git_pillar_privkey': '',
    'git_pillar_pubkey': '',
    'git_pillar_passphrase': '',
    'git_pillar_refspecs': [
        '+refs/heads/*:refs/remotes/origin/*',
        '+refs/tags/*:refs/tags/*',
    ],
    'git_pillar_includes': True,
}
PROC_TIMEOUT = 10
NOTSET = object()


class ProcessManager(object):
    '''
    Functions used both to set up self-contained SSH/HTTP servers for testing
    '''
    wait = 10

    def find_proc(self, name=None, search=None):
        def _search(proc):
            return any([search in x for x in proc.cmdline()])
        if name is None and search is None:
            raise ValueError('one of name or search is required')
        for proc in psutil.process_iter():
            if name is not None:
                if search is None:
                    if name in proc.name():
                        return proc
                elif name in proc.name() and _search(proc):
                    return proc
            else:
                if _search(proc):
                    return proc
        return None

    def wait_proc(self, name=None, search=None, timeout=PROC_TIMEOUT):
        for idx in range(1, self.wait + 1):
            proc = self.find_proc(name=name, search=search)
            if proc is not None:
                return proc
            else:
                if idx != self.wait:
                    log.debug(
                        'Waiting for %s process (%d of %d)',
                        name, idx, self.wait
                    )
                    time.sleep(1)
                else:
                    log.debug(
                        'Failed fo find %s process after %d seconds',
                        name, self.wait
                    )
        raise Exception(
            'Unable to find {0} process running from temp config file {1} '
            'using psutil'.format(name, search)
        )


class SSHDMixin(ModuleCase, ProcessManager, SaltReturnAssertsMixin):
    '''
    Functions to stand up an SSHD server to serve up git repos for tests.
    '''
    sshd_proc = None

    @classmethod
    def prep_server(cls):
        cls.sshd_config_dir = tempfile.mkdtemp(dir=TMP)
        cls.sshd_config = os.path.join(cls.sshd_config_dir, 'sshd_config')
        cls.sshd_port = get_unused_localhost_port()
        cls.url = 'ssh://{username}@127.0.0.1:{port}/~/repo.git'.format(
            username=cls.username,
            port=cls.sshd_port)
        cls.url_extra_repo = 'ssh://{username}@127.0.0.1:{port}/~/extra_repo.git'.format(
            username=cls.username,
            port=cls.sshd_port)
        home = '/root/.ssh'
        cls.ext_opts = {
            'url': cls.url,
            'url_extra_repo': cls.url_extra_repo,
            'privkey_nopass': os.path.join(home, cls.id_rsa_nopass),
            'pubkey_nopass': os.path.join(home, cls.id_rsa_nopass + '.pub'),
            'privkey_withpass': os.path.join(home, cls.id_rsa_withpass),
            'pubkey_withpass': os.path.join(home, cls.id_rsa_withpass + '.pub'),
            'passphrase': cls.passphrase}

    def spawn_server(self):
        ret = self.run_function(
            'state.apply',
            mods='git_pillar.ssh',
            pillar={'git_pillar': {'git_ssh': self.git_ssh,
                                   'id_rsa_nopass': self.id_rsa_nopass,
                                   'id_rsa_withpass': self.id_rsa_withpass,
                                   'sshd_bin': self.sshd_bin,
                                   'sshd_port': self.sshd_port,
                                   'sshd_config_dir': self.sshd_config_dir,
                                   'master_user': self.master_opts['user'],
                                   'user': self.username}}
        )

        try:
            self.sshd_proc = self.wait_proc(name='sshd',
                                            search=self.sshd_config)
        finally:
            # Do the assert after we check for the PID so that we can track
            # it regardless of whether or not something else in the SLS
            # failed (but the SSH server still started).
            self.assertSaltTrueReturn(ret)


class WebserverMixin(ModuleCase, ProcessManager, SaltReturnAssertsMixin):
    '''
    Functions to stand up an nginx + uWSGI + git-http-backend webserver to
    serve up git repos for tests.
    '''
    nginx_proc = uwsgi_proc = None

    @classmethod
    def prep_server(cls):
        '''
        Set up all the webserver paths. Designed to be run once in a
        setUpClass function.
        '''
        cls.root_dir = tempfile.mkdtemp(dir=TMP)
        cls.config_dir = os.path.join(cls.root_dir, 'config')
        cls.nginx_conf = os.path.join(cls.config_dir, 'nginx.conf')
        cls.uwsgi_conf = os.path.join(cls.config_dir, 'uwsgi.yml')
        cls.git_dir = os.path.join(cls.root_dir, 'git')
        cls.repo_dir = os.path.join(cls.git_dir, 'repos')
        cls.venv_dir = os.path.join(cls.root_dir, 'venv')
        cls.uwsgi_bin = os.path.join(cls.venv_dir, 'bin', 'uwsgi')
        cls.nginx_port = cls.uwsgi_port = get_unused_localhost_port()
        while cls.uwsgi_port == cls.nginx_port:
            # Ensure we don't hit a corner case in which two sucessive calls to
            # get_unused_localhost_port() return identical port numbers.
            cls.uwsgi_port = get_unused_localhost_port()
        cls.url = 'http://127.0.0.1:{port}/repo.git'.format(port=cls.nginx_port)
        cls.url_extra_repo = 'http://127.0.0.1:{port}/extra_repo.git'.format(port=cls.nginx_port)
        cls.ext_opts = {'url': cls.url, 'url_extra_repo': cls.url_extra_repo}
        # Add auth params if present (if so this will trigger the spawned
        # server to turn on HTTP basic auth).
        for credential_param in ('user', 'password'):
            if hasattr(cls, credential_param):
                cls.ext_opts[credential_param] = getattr(cls, credential_param)

    @requires_system_grains
    def spawn_server(self, grains):
        auth_enabled = hasattr(self, 'username') and hasattr(self, 'password')
        pillar = {'git_pillar': {'config_dir': self.config_dir,
                                 'git_dir': self.git_dir,
                                 'venv_dir': self.venv_dir,
                                 'root_dir': self.root_dir,
                                 'nginx_port': self.nginx_port,
                                 'uwsgi_port': self.uwsgi_port,
                                 'auth_enabled': auth_enabled}}

        # Different libexec dir for git backend on Debian-based systems
        git_core = '/usr/libexec/git-core' \
            if grains['os_family'] in ('RedHat') \
            else '/usr/lib/git-core'

        pillar['git_pillar']['git-http-backend'] = os.path.join(
            git_core,
            'git-http-backend')

        ret = self.run_function(
            'state.apply',
            mods='git_pillar.http',
            pillar=pillar)

        if not os.path.exists(pillar['git_pillar']['git-http-backend']):
            self.fail(
                '{0} not found. Either git is not installed, or the test '
                'class needs to be updated.'.format(
                    pillar['git_pillar']['git-http-backend']
                )
            )

        try:
            self.nginx_proc = self.wait_proc(name='nginx',
                                             search=self.nginx_conf)
            self.uwsgi_proc = self.wait_proc(name='uwsgi',
                                             search=self.uwsgi_conf)
        finally:
            # Do the assert after we check for the PID so that we can track
            # it regardless of whether or not something else in the SLS
            # failed (but the webserver still started).
            self.assertSaltTrueReturn(ret)


class GitTestBase(ModuleCase):
    '''
    Base class for all gitfs/git_pillar tests. Must be subclassed and paired
    with either SSHDMixin or WebserverMixin to provide the server.
    '''
    case = port = bare_repo = base_extra_repo = admin_repo = admin_extra_repo = None
    maxDiff = None
    git_opts = '-c user.name="Foo Bar" -c user.email=foo@bar.com'
    ext_opts = {}

    # We need to temporarily skip pygit2 tests on EL7 until the EPEL packager
    # updates pygit2 to bring it up-to-date with libgit2.
    @requires_system_grains
    def is_el7(self, grains):
        return grains['os_family'] == 'RedHat' and grains['osmajorrelease'] == 7

    # Cent OS 6 has too old a version of git to handle the make_repo code, as
    # it lacks the -c option for git itself.
    @requires_system_grains
    def is_pre_el7(self, grains):
        return grains['os_family'] == 'RedHat' and grains['osmajorrelease'] < 7

    @classmethod
    def setUpClass(cls):
        cls.prep_server()

    def setUp(self):
        # Make the test class available to the tearDownClass so we can clean up
        # after ourselves. This (and the gated block below) prevent us from
        # needing to spend the extra time creating an ssh server and user and
        # then tear them down separately for each test.
        self.update_class(self)
        if self.is_pre_el7():  # pylint: disable=E1120
            self.skipTest(
                'RHEL < 7 has too old a version of git to run these tests')

    @classmethod
    def update_class(cls, case):
        '''
        Make the test class available to the tearDownClass. Note that this
        cannot be defined in a parent class and inherited, as this will cause
        the parent class to be modified.
        '''
        if getattr(cls, 'case') is None:
            setattr(cls, 'case', case)

    def make_repo(self, root_dir, user='root'):
        raise NotImplementedError()


class GitFSTestBase(GitTestBase, LoaderModuleMockMixin):
    '''
    Base class for all gitfs tests
    '''
    @requires_system_grains
    def setup_loader_modules(self, grains):  # pylint: disable=W0221
        return {
            gitfs: {
                '__opts__': copy.copy(_OPTS),
                '__grains__': grains,
            }
        }

    def make_repo(self, root_dir, user='root'):
        raise NotImplementedError()


class GitPillarTestBase(GitTestBase, LoaderModuleMockMixin):
    '''
    Base class for all git_pillar tests
    '''
    @requires_system_grains
    def setup_loader_modules(self, grains):  # pylint: disable=W0221
        return {
            git_pillar: {
                '__opts__': copy.copy(_OPTS),
                '__grains__': grains,
            }
        }

    def get_pillar(self, ext_pillar_conf):
        '''
        Run git_pillar with the specified configuration
        '''
        cachedir = tempfile.mkdtemp(dir=TMP)
        self.addCleanup(shutil.rmtree, cachedir, ignore_errors=True)
        ext_pillar_opts = {'optimization_order': [0, 1, 2]}
        ext_pillar_opts.update(
            salt.utils.yaml.safe_load(
                ext_pillar_conf.format(
                    cachedir=cachedir,
                    extmods=os.path.join(cachedir, 'extmods'),
                    **self.ext_opts
                )
            )
        )
        with patch.dict(git_pillar.__opts__, ext_pillar_opts):
            return git_pillar.ext_pillar(
                'minion',
                {},
                *ext_pillar_opts['ext_pillar'][0]['git']
            )

    def make_repo(self, root_dir, user='root'):
        self.bare_repo = os.path.join(root_dir, 'repo.git')
        self.admin_repo = os.path.join(root_dir, 'admin')

        for dirname in (self.bare_repo, self.admin_repo):
            shutil.rmtree(dirname, ignore_errors=True)

        # Create bare repo
        self.run_function(
            'git.init',
            [self.bare_repo],
            user=user,
            bare=True)

        # Clone bare repo
        self.run_function(
            'git.clone',
            [self.admin_repo],
            url=self.bare_repo,
            user=user)

        def _push(branch, message):
            self.run_function(
                'git.add',
                [self.admin_repo, '.'],
                user=user)
            self.run_function(
                'git.commit',
                [self.admin_repo, message],
                user=user,
                git_opts=self.git_opts,
            )
            self.run_function(
                'git.push',
                [self.admin_repo],
                remote='origin',
                ref=branch,
                user=user,
            )

        with salt.utils.files.fopen(
                os.path.join(self.admin_repo, 'top.sls'), 'w') as fp_:
            fp_.write(textwrap.dedent('''\
            base:
              '*':
                - foo
            '''))
        with salt.utils.files.fopen(
                os.path.join(self.admin_repo, 'foo.sls'), 'w') as fp_:
            fp_.write(textwrap.dedent('''\
            branch: master
            mylist:
              - master
            mydict:
              master: True
              nested_list:
                - master
              nested_dict:
                master: True
            '''))
        # Add another file to be referenced using git_pillar_includes
        with salt.utils.files.fopen(
                os.path.join(self.admin_repo, 'bar.sls'), 'w') as fp_:
            fp_.write('included_pillar: True\n')
        # Add another file in subdir
        os.mkdir(os.path.join(self.admin_repo, 'subdir'))
        with salt.utils.files.fopen(
                os.path.join(self.admin_repo, 'subdir', 'bar.sls'), 'w') as fp_:
            fp_.write('from_subdir: True\n')
        _push('master', 'initial commit')

        # Do the same with different values for "dev" branch
        self.run_function(
            'git.checkout',
            [self.admin_repo],
            user=user,
            opts='-b dev')
        # The bar.sls shouldn't be in any branch but master
        self.run_function(
            'git.rm',
            [self.admin_repo, 'bar.sls'],
            user=user)
        with salt.utils.files.fopen(
                os.path.join(self.admin_repo, 'top.sls'), 'w') as fp_:
            fp_.write(textwrap.dedent('''\
            dev:
              '*':
                - foo
            '''))
        with salt.utils.files.fopen(
                os.path.join(self.admin_repo, 'foo.sls'), 'w') as fp_:
            fp_.write(textwrap.dedent('''\
            branch: dev
            mylist:
              - dev
            mydict:
              dev: True
              nested_list:
                - dev
              nested_dict:
                dev: True
            '''))
        _push('dev', 'add dev branch')

        # Create just a top file in a separate repo, to be mapped to the base
        # env and referenced using git_pillar_includes
        self.run_function(
            'git.checkout',
            [self.admin_repo],
            user=user,
            opts='-b top_only')
        # The top.sls should be the only file in this branch
        self.run_function(
            'git.rm',
            [self.admin_repo, 'foo.sls', os.path.join('subdir', 'bar.sls')],
            user=user)
        with salt.utils.files.fopen(
                os.path.join(self.admin_repo, 'top.sls'), 'w') as fp_:
            fp_.write(textwrap.dedent('''\
            base:
              '*':
                - bar
            '''))
        _push('top_only', 'add top_only branch')

        # Create just another top file in a separate repo, to be mapped to the base
        # env and including mounted.bar
        self.run_function(
            'git.checkout',
            [self.admin_repo],
            user=user,
            opts='-b top_mounted')
        # The top.sls should be the only file in this branch
        with salt.utils.files.fopen(
                os.path.join(self.admin_repo, 'top.sls'), 'w') as fp_:
            fp_.write(textwrap.dedent('''\
            base:
              '*':
                - mounted.bar
            '''))
        _push('top_mounted', 'add top_mounted branch')

    def make_extra_repo(self, root_dir, user='root'):
        self.bare_extra_repo = os.path.join(root_dir, 'extra_repo.git')
        self.admin_extra_repo = os.path.join(root_dir, 'admin_extra')

        for dirname in (self.bare_extra_repo, self.admin_extra_repo):
            shutil.rmtree(dirname, ignore_errors=True)

        # Create bare extra repo
        self.run_function(
            'git.init',
            [self.bare_extra_repo],
            user=user,
            bare=True)

        # Clone bare repo
        self.run_function(
            'git.clone',
            [self.admin_extra_repo],
            url=self.bare_extra_repo,
            user=user)

        def _push(branch, message):
            self.run_function(
                'git.add',
                [self.admin_extra_repo, '.'],
                user=user)
            self.run_function(
                'git.commit',
                [self.admin_extra_repo, message],
                user=user,
                git_opts=self.git_opts,
            )
            self.run_function(
                'git.push',
                [self.admin_extra_repo],
                remote='origin',
                ref=branch,
                user=user,
            )

        with salt.utils.files.fopen(
                os.path.join(self.admin_extra_repo, 'top.sls'), 'w') as fp_:
            fp_.write(textwrap.dedent('''\
            "{{saltenv}}":
              '*':
                - motd
                - nowhere.foo
            '''))
        with salt.utils.files.fopen(
                os.path.join(self.admin_extra_repo, 'motd.sls'), 'w') as fp_:
            fp_.write(textwrap.dedent('''\
            motd: The force will be with you. Always.
            '''))
        _push('master', 'initial commit')


class GitPillarSSHTestBase(GitPillarTestBase, SSHDMixin):
    '''
    Base class for GitPython and Pygit2 SSH tests
    '''
    id_rsa_nopass = id_rsa_withpass = None
    git_ssh = '/tmp/git_ssh'

    @classmethod
    def tearDownClass(cls):
        if cls.case is None:
            return
        if cls.case.sshd_proc is not None:
            cls.case.sshd_proc.send_signal(signal.SIGTERM)
        cls.case.run_state('user.absent', name=cls.username, purge=True)
        for dirname in (cls.sshd_config_dir, cls.case.admin_repo,
                        cls.case.bare_repo):
            if dirname is not None:
                shutil.rmtree(dirname, ignore_errors=True)
        ssh_dir = os.path.expanduser('~/.ssh')
        for filename in (cls.id_rsa_nopass,
                         cls.id_rsa_nopass + '.pub',
                         cls.id_rsa_withpass,
                         cls.id_rsa_withpass + '.pub',
                         cls.git_ssh):
            try:
                os.remove(os.path.join(ssh_dir, filename))
            except OSError as exc:
                if exc.errno != errno.ENOENT:
                    raise

    def setUp(self):
        '''
        Create the SSH server and user, and create the git repo
        '''
        super(GitPillarSSHTestBase, self).setUp()
        self.sshd_proc = self.find_proc(name='sshd',
                                        search=self.sshd_config)
        self.sshd_bin = salt.utils.path.which('sshd')

        if self.sshd_proc is None:
            self.spawn_server()

            known_hosts_ret = self.run_function(
                'ssh.set_known_host',
                user=self.master_opts['user'],
                hostname='127.0.0.1',
                port=self.sshd_port,
                enc='ssh-rsa',
                fingerprint='fd:6f:7f:5d:06:6b:f2:06:0d:26:93:9e:5a:b5:19:46',
                hash_known_hosts=False,
                fingerprint_hash_type='md5',
            )
            if 'error' in known_hosts_ret:
                raise Exception(
                    'Failed to add key to {0} user\'s known_hosts '
                    'file: {1}'.format(
                        self.master_opts['user'],
                        known_hosts_ret['error']
                    )
                )

        root_dir = os.path.expanduser('~{0}'.format(self.username))
        if root_dir.startswith('~'):
            self.fail(
                'Unable to resolve homedir for user \'{0}\''.format(
                    self.username
                )
            )
        self.make_repo(root_dir, user=self.username)
        self.make_extra_repo(root_dir, user=self.username)

    def get_pillar(self, ext_pillar_conf):
        '''
        Wrap the parent class' get_pillar() func in logic that temporarily
        changes the GIT_SSH to use our custom script, ensuring that the
        passphraselsess key is used to auth without needing to modify the root
        user's ssh config file.
        '''
        orig_git_ssh = os.environ.pop('GIT_SSH', NOTSET)
        os.environ['GIT_SSH'] = self.git_ssh
        try:
            return super(GitPillarSSHTestBase, self).get_pillar(ext_pillar_conf)
        finally:
            os.environ.pop('GIT_SSH', None)
            if orig_git_ssh is not NOTSET:
                os.environ['GIT_SSH'] = orig_git_ssh


class GitPillarHTTPTestBase(GitPillarTestBase, WebserverMixin):
    '''
    Base class for GitPython and Pygit2 HTTP tests
    '''
    @classmethod
    def tearDownClass(cls):
        for proc in (cls.case.nginx_proc, cls.case.uwsgi_proc):
            if proc is not None:
                try:
                    proc.send_signal(signal.SIGQUIT)
                except psutil.NoSuchProcess:
                    pass
        shutil.rmtree(cls.root_dir, ignore_errors=True)

    def setUp(self):
        '''
        Create and start the webserver, and create the git repo
        '''
        super(GitPillarHTTPTestBase, self).setUp()
        self.nginx_proc = self.find_proc(name='nginx',
                                         search=self.nginx_conf)
        self.uwsgi_proc = self.find_proc(name='uwsgi',
                                         search=self.uwsgi_conf)

        if self.nginx_proc is None and self.uwsgi_proc is None:
            self.spawn_server()  # pylint: disable=E1120

        self.make_repo(self.repo_dir)
        self.make_extra_repo(self.repo_dir)



# -*- coding: utf-8 -*-
'''
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
'''

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import NO_MOCK, NO_MOCK_REASON

# Import Salt Libs
import salt.modules.haproxyconn as haproxyconn


class Mockcmds(object):
    '''
    Mock of cmds
    '''
    def __init__(self):
        self.backend = None
        self.server = None
        self.weight = None

    def listServers(self, backend):
        '''
        Mock of listServers method
        '''
        self.backend = backend
        return 'Name: server01 Status: UP Weight: 1 bIn: 22 bOut: 12\n' \
               'Name: server02 Status: MAINT Weight: 2 bIn: 0 bOut: 0'

    def enableServer(self, server, backend):
        '''
        Mock of enableServer method
        '''
        self.backend = backend
        self.server = server
        return 'server enabled'

    def disableServer(self, server, backend):
        '''
        Mock of disableServer method
        '''
        self.backend = backend
        self.server = server
        return 'server disabled'

    def getWeight(self, server, backend, weight=0):
        '''
        Mock of getWeight method
        '''
        self.backend = backend
        self.server = server
        self.weight = weight
        return 'server weight'

    @staticmethod
    def showFrontends():
        '''
        Mock of showFrontends method
        '''
        return 'frontend-alpha\n' \
               'frontend-beta\n' \
               'frontend-gamma'

    @staticmethod
    def showBackends():
        '''
        Mock of showBackends method
        '''
        return 'backend-alpha\n' \
               'backend-beta\n' \
               'backend-gamma'


class Mockhaproxy(object):
    '''
    Mock of haproxy
    '''
    def __init__(self):
        self.cmds = Mockcmds()


class MockHaConn(object):
    '''
    Mock of HaConn
    '''
    def __init__(self, socket=None):
        self.ha_cmd = None

    def sendCmd(self, ha_cmd, objectify=False):
        '''
        Mock of sendCmd method
        '''
        self.ha_cmd = ha_cmd
        self.objectify = objectify
        return ha_cmd


@skipIf(NO_MOCK, NO_MOCK_REASON)
class HaproxyConnTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.modules.haproxyconn
    '''
    def setup_loader_modules(self):
        return {haproxyconn: {'haproxy': Mockhaproxy(), '_get_conn': MockHaConn}}

    # 'list_servers' function tests: 1

    def test_list_servers(self):
        '''
        Test list_servers
        '''
        self.assertTrue(haproxyconn.list_servers('mysql'))

    # 'enable_server' function tests: 1

    def test_enable_server(self):
        '''
        Test enable_server
        '''
        self.assertTrue(haproxyconn.enable_server('web1.salt.com', 'www'))

    # 'disable_server' function tests: 1

    def test_disable_server(self):
        '''
        Test disable_server
        '''
        self.assertTrue(haproxyconn.disable_server('db1.salt.com', 'mysql'))

    # 'get_weight' function tests: 1

    def test_get_weight(self):
        '''
        Test get the weight of a server
        '''
        self.assertTrue(haproxyconn.get_weight('db1.salt.com', 'mysql'))

    # 'set_weight' function tests: 1

    def test_set_weight(self):
        '''
        Test setting the weight of a given server
        '''
        self.assertTrue(haproxyconn.set_weight('db1.salt.com', 'mysql',
                                               weight=11))

    # 'show_frontends' function tests: 1

    def test_show_frontends(self):
        '''
        Test print all frontends received from the HAProxy socket
        '''
        self.assertTrue(haproxyconn.show_frontends())

    def test_list_frontends(self):
        '''
        Test listing all frontends
        '''
        self.assertEqual(
            sorted(haproxyconn.list_frontends()),
            sorted(['frontend-alpha', 'frontend-beta', 'frontend-gamma'])
        )

    # 'show_backends' function tests: 1

    def test_show_backends(self):
        '''
        Test print all backends received from the HAProxy socket
        '''
        self.assertTrue(haproxyconn.show_backends())

    def test_list_backends(self):
        '''
        Test listing of all backends
        '''
        self.assertEqual(
            sorted(haproxyconn.list_backends()),
            sorted(['backend-alpha', 'backend-beta', 'backend-gamma'])
        )

    def test_get_backend(self):
        '''
        Test get_backend and compare returned value
        '''
        expected_data = {
            'server01': {
                'status': 'UP',
                'weight': 1,
                'bin': 22,
                'bout': 12
            },
            'server02': {
                'status': 'MAINT',
                'weight': 2,
                'bin': 0,
                'bout': 0
            }
        }
        self.assertDictEqual(haproxyconn.get_backend('test'), expected_data)

    def test_wait_state_true(self):
        '''
        Test a successful wait for state
        '''
        self.assertTrue(haproxyconn.wait_state('test', 'server01'))

    def test_wait_state_false(self):
        '''
        Test a failed wait for state, with a timeout of 0
        '''
        self.assertFalse(haproxyconn.wait_state('test', 'server02', 'up', 0))



# -*- coding: utf-8 -*-

# Import python libs
from __future__ import absolute_import, unicode_literals
import sys

# Import Salt Libs
import salt.utils.minions

# Import Salt Testing Libs
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    patch,
    MagicMock,
)

NODEGROUPS = {
    'group1': 'L@host1,host2,host3',
    'group2': ['G@foo:bar', 'or', 'web1*'],
    'group3': ['N@group1', 'or', 'N@group2'],
    'group4': ['host4', 'host5', 'host6'],
}

EXPECTED = {
    'group1': ['L@host1,host2,host3'],
    'group2': ['G@foo:bar', 'or', 'web1*'],
    'group3': ['(', '(', 'L@host1,host2,host3', ')', 'or', '(', 'G@foo:bar', 'or', 'web1*', ')', ')'],
    'group4': ['L@host4,host5,host6'],
}


class MinionsTestCase(TestCase):
    '''
    TestCase for salt.utils.minions module functions
    '''
    def test_nodegroup_comp(self):
        '''
        Test a simple string nodegroup
        '''
        for nodegroup in NODEGROUPS:
            expected = EXPECTED[nodegroup]
            ret = salt.utils.minions.nodegroup_comp(nodegroup, NODEGROUPS)
            self.assertEqual(ret, expected)


class CkMinionsTestCase(TestCase):
    '''
    TestCase for salt.utils.minions.CkMinions class
    '''
    def setUp(self):
        self.ckminions = salt.utils.minions.CkMinions({'minion_data_cache': True})

    def test_spec_check(self):
        # Test spec-only rule
        auth_list = ['@runner']
        ret = self.ckminions.spec_check(auth_list, 'test.arg', {}, 'runner')
        self.assertTrue(ret)
        ret = self.ckminions.spec_check(auth_list, 'test.arg', {}, 'wheel')
        self.assertFalse(ret)
        ret = self.ckminions.spec_check(auth_list, 'testarg', {}, 'runner')
        mock_ret = {'error': {'name': 'SaltInvocationError',
                              'message': 'A command invocation error occurred: Check syntax.'}}
        self.assertDictEqual(mock_ret, ret)

        # Test spec in plural form
        auth_list = ['@runners']
        ret = self.ckminions.spec_check(auth_list, 'test.arg', {}, 'runner')
        self.assertTrue(ret)
        ret = self.ckminions.spec_check(auth_list, 'test.arg', {}, 'wheel')
        self.assertFalse(ret)

        # Test spec with module.function restriction
        auth_list = [{'@runner': 'test.arg'}]
        ret = self.ckminions.spec_check(auth_list, 'test.arg', {}, 'runner')
        self.assertTrue(ret)
        ret = self.ckminions.spec_check(auth_list, 'test.arg', {}, 'wheel')
        self.assertFalse(ret)
        ret = self.ckminions.spec_check(auth_list, 'tes.arg', {}, 'runner')
        self.assertFalse(ret)
        ret = self.ckminions.spec_check(auth_list, 'test.ar', {}, 'runner')
        self.assertFalse(ret)

        # Test function name is a regex
        auth_list = [{'@runner': 'test.arg.*some'}]
        ret = self.ckminions.spec_check(auth_list, 'test.arg', {}, 'runner')
        self.assertFalse(ret)
        ret = self.ckminions.spec_check(auth_list, 'test.argsome', {}, 'runner')
        self.assertTrue(ret)
        ret = self.ckminions.spec_check(auth_list, 'test.arg_aaa_some', {}, 'runner')
        self.assertTrue(ret)

        # Test a list of funcs
        auth_list = [{'@runner': ['test.arg', 'jobs.active']}]
        ret = self.ckminions.spec_check(auth_list, 'test.arg', {}, 'runner')
        self.assertTrue(ret)
        ret = self.ckminions.spec_check(auth_list, 'jobs.active', {}, 'runner')
        self.assertTrue(ret)
        ret = self.ckminions.spec_check(auth_list, 'test.active', {}, 'runner')
        self.assertFalse(ret)
        ret = self.ckminions.spec_check(auth_list, 'jobs.arg', {}, 'runner')
        self.assertFalse(ret)

        # Test args-kwargs rules
        auth_list = [{
                '@runner': {
                    'test.arg': {
                        'args': ['1', '2'],
                        'kwargs': {
                            'aaa': 'bbb',
                            'ccc': 'ddd'
                            }
                        }
                    }
                }]
        ret = self.ckminions.spec_check(auth_list, 'test.arg', {}, 'runner')
        self.assertFalse(ret)
        args = {
                'arg': ['1', '2'],
                'kwarg': {'aaa': 'bbb', 'ccc': 'ddd'}
                }
        ret = self.ckminions.spec_check(auth_list, 'test.arg', args, 'runner')
        self.assertTrue(ret)
        args = {
                'arg': ['1', '2', '3'],
                'kwarg': {'aaa': 'bbb', 'ccc': 'ddd'}
                }
        ret = self.ckminions.spec_check(auth_list, 'test.arg', args, 'runner')
        self.assertTrue(ret)
        args = {
                'arg': ['1', '2'],
                'kwarg': {'aaa': 'bbb', 'ccc': 'ddd', 'zzz': 'zzz'}
                }
        ret = self.ckminions.spec_check(auth_list, 'test.arg', args, 'runner')
        self.assertTrue(ret)
        args = {
                'arg': ['1', '2'],
                'kwarg': {'aaa': 'bbb', 'ccc': 'ddc'}
                }
        ret = self.ckminions.spec_check(auth_list, 'test.arg', args, 'runner')
        self.assertFalse(ret)
        args = {
                'arg': ['1', '2'],
                'kwarg': {'aaa': 'bbb'}
                }
        ret = self.ckminions.spec_check(auth_list, 'test.arg', args, 'runner')
        self.assertFalse(ret)
        args = {
                'arg': ['1', '3'],
                'kwarg': {'aaa': 'bbb', 'ccc': 'ddd'}
                }
        ret = self.ckminions.spec_check(auth_list, 'test.arg', args, 'runner')
        self.assertFalse(ret)
        args = {
                'arg': ['1'],
                'kwarg': {'aaa': 'bbb', 'ccc': 'ddd'}
                }
        ret = self.ckminions.spec_check(auth_list, 'test.arg', args, 'runner')
        self.assertFalse(ret)
        args = {
                'kwarg': {'aaa': 'bbb', 'ccc': 'ddd'}
                }
        ret = self.ckminions.spec_check(auth_list, 'test.arg', args, 'runner')
        self.assertFalse(ret)
        args = {
                'arg': ['1', '2'],
                }
        ret = self.ckminions.spec_check(auth_list, 'test.arg', args, 'runner')
        self.assertFalse(ret)

        # Test kwargs only
        auth_list = [{
                '@runner': {
                    'test.arg': {
                        'kwargs': {
                            'aaa': 'bbb',
                            'ccc': 'ddd'
                            }
                        }
                    }
                }]
        ret = self.ckminions.spec_check(auth_list, 'test.arg', {}, 'runner')
        self.assertFalse(ret)
        args = {
                'arg': ['1', '2'],
                'kwarg': {'aaa': 'bbb', 'ccc': 'ddd'}
                }
        ret = self.ckminions.spec_check(auth_list, 'test.arg', args, 'runner')
        self.assertTrue(ret)

        # Test args only
        auth_list = [{
                '@runner': {
                    'test.arg': {
                        'args': ['1', '2']
                        }
                    }
                }]
        ret = self.ckminions.spec_check(auth_list, 'test.arg', {}, 'runner')
        self.assertFalse(ret)
        args = {
                'arg': ['1', '2'],
                'kwarg': {'aaa': 'bbb', 'ccc': 'ddd'}
                }
        ret = self.ckminions.spec_check(auth_list, 'test.arg', args, 'runner')
        self.assertTrue(ret)

        # Test list of args
        auth_list = [{'@runner': [{'test.arg': [{'args': ['1', '2'],
                                                 'kwargs': {'aaa': 'bbb',
                                                            'ccc': 'ddd'
                                                            }
                                                 },
                                                {'args': ['2', '3'],
                                                 'kwargs': {'aaa': 'aaa',
                                                            'ccc': 'ccc'
                                                            }
                                                 }]
                                   }]
                      }]
        args = {
                'arg': ['1', '2'],
                'kwarg': {'aaa': 'bbb', 'ccc': 'ddd'}
                }
        ret = self.ckminions.spec_check(auth_list, 'test.arg', args, 'runner')
        self.assertTrue(ret)
        args = {
                'arg': ['2', '3'],
                'kwarg': {'aaa': 'aaa', 'ccc': 'ccc'}
                }
        ret = self.ckminions.spec_check(auth_list, 'test.arg', args, 'runner')
        self.assertTrue(ret)

        # Test @module form
        auth_list = ['@jobs']
        ret = self.ckminions.spec_check(auth_list, 'jobs.active', {}, 'runner')
        self.assertTrue(ret)
        ret = self.ckminions.spec_check(auth_list, 'jobs.active', {}, 'wheel')
        self.assertTrue(ret)
        ret = self.ckminions.spec_check(auth_list, 'test.arg', {}, 'runner')
        self.assertFalse(ret)
        ret = self.ckminions.spec_check(auth_list, 'job.arg', {}, 'runner')
        self.assertFalse(ret)

        # Test @module: function
        auth_list = [{'@jobs': 'active'}]
        ret = self.ckminions.spec_check(auth_list, 'jobs.active', {}, 'runner')
        self.assertTrue(ret)
        ret = self.ckminions.spec_check(auth_list, 'jobs.active', {}, 'wheel')
        self.assertTrue(ret)
        ret = self.ckminions.spec_check(auth_list, 'jobs.active_jobs', {}, 'runner')
        self.assertTrue(ret)
        ret = self.ckminions.spec_check(auth_list, 'jobs.activ', {}, 'runner')
        self.assertFalse(ret)

        # Test @module: [functions]
        auth_list = [{'@jobs': ['active', 'li']}]
        ret = self.ckminions.spec_check(auth_list, 'jobs.active', {}, 'runner')
        self.assertTrue(ret)
        ret = self.ckminions.spec_check(auth_list, 'jobs.list_jobs', {}, 'runner')
        self.assertTrue(ret)
        ret = self.ckminions.spec_check(auth_list, 'jobs.last_run', {}, 'runner')
        self.assertFalse(ret)

        # Test @module: function with args
        auth_list = [{'@jobs': {'active': {'args': ['1', '2'],
                                           'kwargs': {'a': 'b', 'c': 'd'}}}}]
        args = {'arg': ['1', '2'],
                'kwarg': {'a': 'b', 'c': 'd'}}
        ret = self.ckminions.spec_check(auth_list, 'jobs.active', args, 'runner')
        self.assertTrue(ret)
        ret = self.ckminions.spec_check(auth_list, 'jobs.active', {}, 'runner')
        self.assertFalse(ret)

    @patch('salt.utils.minions.CkMinions._pki_minions', MagicMock(return_value=['alpha', 'beta', 'gamma']))
    def test_auth_check(self):
        # Test function-only rule
        auth_list = ['test.ping']
        ret = self.ckminions.auth_check(auth_list, 'test.ping', None, 'alpha')
        self.assertTrue(ret)
        ret = self.ckminions.auth_check(auth_list, 'test.arg', None, 'alpha')
        self.assertFalse(ret)

        # Test minion and function
        auth_list = [{'alpha': 'test.ping'}]
        ret = self.ckminions.auth_check(auth_list, 'test.ping', None, 'alpha')
        self.assertTrue(ret)
        ret = self.ckminions.auth_check(auth_list, 'test.arg', None, 'alpha')
        self.assertFalse(ret)
        ret = self.ckminions.auth_check(auth_list, 'test.ping', None, 'beta')
        self.assertFalse(ret)

        # Test function list
        auth_list = [{'*': ['test.*', 'saltutil.cmd']}]
        ret = self.ckminions.auth_check(auth_list, 'test.arg', None, 'alpha')
        self.assertTrue(ret)
        ret = self.ckminions.auth_check(auth_list, 'test.ping', None, 'beta')
        self.assertTrue(ret)
        ret = self.ckminions.auth_check(auth_list, 'saltutil.cmd', None, 'gamma')
        self.assertTrue(ret)
        ret = self.ckminions.auth_check(auth_list, 'saltutil.running', None, 'gamma')
        self.assertFalse(ret)

        # Test an args and kwargs rule
        auth_list = [{
                'alpha': {
                    'test.arg': {
                        'args': ['1', '2'],
                        'kwargs': {
                            'aaa': 'bbb',
                            'ccc': 'ddd'
                            }
                        }
                    }
                }]
        ret = self.ckminions.auth_check(auth_list, 'test.arg', None, 'runner')
        self.assertFalse(ret)
        ret = self.ckminions.auth_check(auth_list, 'test.arg', [], 'runner')
        self.assertFalse(ret)
        args = ['1', '2', {'aaa': 'bbb', 'ccc': 'ddd', '__kwarg__': True}]
        ret = self.ckminions.auth_check(auth_list, 'test.arg', args, 'runner')
        self.assertTrue(ret)
        args = ['1', '2', '3', {'aaa': 'bbb', 'ccc': 'ddd', 'eee': 'fff', '__kwarg__': True}]
        ret = self.ckminions.auth_check(auth_list, 'test.arg', args, 'runner')
        self.assertTrue(ret)
        args = ['1', {'aaa': 'bbb', 'ccc': 'ddd', '__kwarg__': True}]
        ret = self.ckminions.auth_check(auth_list, 'test.arg', args, 'runner')
        self.assertFalse(ret)
        args = ['1', '2', {'aaa': 'bbb', '__kwarg__': True}]
        ret = self.ckminions.auth_check(auth_list, 'test.arg', args, 'runner')
        self.assertFalse(ret)
        args = ['1', '3', {'aaa': 'bbb', 'ccc': 'ddd', '__kwarg__': True}]
        ret = self.ckminions.auth_check(auth_list, 'test.arg', args, 'runner')
        self.assertFalse(ret)
        args = ['1', '2', {'aaa': 'bbb', 'ccc': 'fff', '__kwarg__': True}]
        ret = self.ckminions.auth_check(auth_list, 'test.arg', args, 'runner')
        self.assertFalse(ret)

        # Test kwargs only rule
        auth_list = [{
                'alpha': {
                    'test.arg': {
                        'kwargs': {
                            'aaa': 'bbb',
                            'ccc': 'ddd'
                            }
                        }
                    }
                }]
        args = ['1', '2', {'aaa': 'bbb', 'ccc': 'ddd', '__kwarg__': True}]
        ret = self.ckminions.auth_check(auth_list, 'test.arg', args, 'runner')
        self.assertTrue(ret)
        args = [{'aaa': 'bbb', 'ccc': 'ddd', 'eee': 'fff', '__kwarg__': True}]
        ret = self.ckminions.auth_check(auth_list, 'test.arg', args, 'runner')
        self.assertTrue(ret)

        # Test args only rule
        auth_list = [{
                'alpha': {
                    'test.arg': {
                        'args': ['1', '2'],
                        }
                    }
                }]
        args = ['1', '2', {'aaa': 'bbb', 'ccc': 'ddd', '__kwarg__': True}]
        ret = self.ckminions.auth_check(auth_list, 'test.arg', args, 'runner')
        self.assertTrue(ret)
        args = ['1', '2']
        ret = self.ckminions.auth_check(auth_list, 'test.arg', args, 'runner')
        self.assertTrue(ret)


@skipIf(sys.version_info < (2, 7), 'Python 2.7 needed for dictionary equality assertions')
class TargetParseTestCase(TestCase):

    def test_parse_grains_target(self):
        '''
        Ensure proper parsing for grains
        '''
        g_tgt = 'G@a:b'
        ret = salt.utils.minions.parse_target(g_tgt)
        self.assertDictEqual(ret, {'engine': 'G', 'pattern': 'a:b', 'delimiter': None})

    def test_parse_grains_pcre_target(self):
        '''
        Ensure proper parsing for grains PCRE matching
        '''
        p_tgt = 'P@a:b'
        ret = salt.utils.minions.parse_target(p_tgt)
        self.assertDictEqual(ret, {'engine': 'P', 'pattern': 'a:b', 'delimiter': None})

    def test_parse_pillar_pcre_target(self):
        '''
        Ensure proper parsing for pillar PCRE matching
        '''
        j_tgt = 'J@a:b'
        ret = salt.utils.minions.parse_target(j_tgt)
        self.assertDictEqual(ret, {'engine': 'J', 'pattern': 'a:b', 'delimiter': None})

    def test_parse_list_target(self):
        '''
        Ensure proper parsing for list matching
        '''
        l_tgt = 'L@a:b'
        ret = salt.utils.minions.parse_target(l_tgt)
        self.assertDictEqual(ret, {'engine': 'L', 'pattern': 'a:b', 'delimiter': None})

    def test_parse_nodegroup_target(self):
        '''
        Ensure proper parsing for pillar matching
        '''
        n_tgt = 'N@a:b'
        ret = salt.utils.minions.parse_target(n_tgt)
        self.assertDictEqual(ret, {'engine': 'N', 'pattern': 'a:b', 'delimiter': None})

    def test_parse_subnet_target(self):
        '''
        Ensure proper parsing for subnet matching
        '''
        s_tgt = 'S@a:b'
        ret = salt.utils.minions.parse_target(s_tgt)
        self.assertDictEqual(ret, {'engine': 'S', 'pattern': 'a:b', 'delimiter': None})

    def test_parse_minion_pcre_target(self):
        '''
        Ensure proper parsing for minion PCRE matching
        '''
        e_tgt = 'E@a:b'
        ret = salt.utils.minions.parse_target(e_tgt)
        self.assertDictEqual(ret, {'engine': 'E', 'pattern': 'a:b', 'delimiter': None})

    def test_parse_range_target(self):
        '''
        Ensure proper parsing for range matching
        '''
        r_tgt = 'R@a:b'
        ret = salt.utils.minions.parse_target(r_tgt)
        self.assertDictEqual(ret, {'engine': 'R', 'pattern': 'a:b', 'delimiter': None})

    def test_parse_multiword_target(self):
        '''
        Ensure proper parsing for multi-word targets

        Refs https://github.com/saltstack/salt/issues/37231
        '''
        mw_tgt = 'G@a:b c'
        ret = salt.utils.minions.parse_target(mw_tgt)
        self.assertEqual(ret['pattern'], 'a:b c')


class NodegroupCompTest(TestCase):
    '''
    Test nodegroup comparisons found in
    salt.utils.minions.nodgroup_comp()
    '''

    def test_simple_nodegroup(self):
        '''
        Smoke test a very simple nodegroup. No recursion.
        '''
        simple_nodegroup = {'group1': 'L@foo.domain.com,bar.domain.com,baz.domain.com or bl*.domain.com'}

        ret = salt.utils.minions.nodegroup_comp('group1', simple_nodegroup)
        expected_ret = ['L@foo.domain.com,bar.domain.com,baz.domain.com', 'or', 'bl*.domain.com']
        self.assertListEqual(ret, expected_ret)

    def test_simple_expression_nodegroup(self):
        '''
        Smoke test a nodegroup with a simple expression. No recursion.
        '''
        simple_nodegroup = {'group1': '[foo,bar,baz].domain.com'}

        ret = salt.utils.minions.nodegroup_comp('group1', simple_nodegroup)
        expected_ret = ['E@[foo,bar,baz].domain.com']
        self.assertListEqual(ret, expected_ret)

    def test_simple_recurse(self):
        '''
        Test a case where one nodegroup contains a second nodegroup
        '''
        referenced_nodegroups = {
                'group1': 'L@foo.domain.com,bar.domain.com,baz.domain.com or bl*.domain.com',
                'group2': 'G@os:Debian and N@group1'
                }

        ret = salt.utils.minions.nodegroup_comp('group2', referenced_nodegroups)
        expected_ret = [
                '(',
                'G@os:Debian',
                'and',
                '(',
                'L@foo.domain.com,bar.domain.com,baz.domain.com',
                'or',
                'bl*.domain.com',
                ')',
                ')'
                ]
        self.assertListEqual(ret, expected_ret)

    def test_circular_nodegroup_reference(self):
        '''
        Test to see what happens if A refers to B
        and B in turn refers back to A
        '''
        referenced_nodegroups = {
                'group1': 'N@group2',
                'group2': 'N@group1'
                }

        # If this works, it should also print an error to the console
        ret = salt.utils.minions.nodegroup_comp('group1', referenced_nodegroups)
        self.assertEqual(ret, [])
